@@ -51,4 +51,17 @@
 #define be64toh(x) OSSwapBigToHostInt64(x)
 #endif
 
+#define FLB_LITTLE_ENDIAN 0
+#define FLB_BIG_ENDIAN    1
+
+#ifndef FLB_BYTE_ORDER
+    #if defined(__BYTE_ORDER__) &&  __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+        #define FLB_BYTE_ORDER FLB_BIG_ENDIAN
+    #elif defined(__BIG_ENDIAN__) || defined(__BIG_ENDIAN) || defined(_BIG_ENDIAN)
+        #define FLB_BYTE_ORDER FLB_BIG_ENDIAN
+    #else
+        #define FLB_BYTE_ORDER FLB_LITTLE_ENDIAN
+    #endif
+#endif
+
 #endif
