@@ -32,7 +32,9 @@
 #include <unistd.h>
 #endif
 #include <fcntl.h>
-
+#ifdef HAVE_SYS_IOCTL_H
+#  include <sys/ioctl.h>
+#endif
 extern "C" {
 // Remove command-line defines of package variables for the test project...
 #undef PACKAGE_NAME
@@ -40,11 +42,10 @@ extern "C" {
 #undef PACKAGE_STRING
 #undef PACKAGE_TARNAME
 // ... so we can include the library's config without symbol redefinitions.
-#include "ares_setup.h"
+#include "ares_private.h"
 #include "ares_inet_net_pton.h"
 #include "ares_data.h"
 #include "ares_strsplit.h"
-#include "ares_private.h"
 #include "ares__htable.h"
 
 #ifdef HAVE_ARPA_INET_H
@@ -235,7 +236,7 @@ TEST_F(LibraryTest, FreeCorruptData) {
   void* p = &(data->data);
 
   // Invalid type
-  data->type = (ares_datatype)99;
+  data->type = (ares_datatype)ARES_DATATYPE_LAST;
   data->mark = ARES_DATATYPE_MARK;
   ares_free_data(p);
 
@@ -282,49 +283,6 @@ TEST_F(LibraryTest, MallocDataFail) {
   EXPECT_EQ(nullptr, ares_malloc_data(ARES_DATATYPE_MX_REPLY));
 }
 
-TEST_F(LibraryTest, ReadLine) {
-  TempFile temp("abcde\n0123456789\nXYZ\n012345678901234567890\n\n");
-  FILE *fp = fopen(temp.filename(), "r");
-  size_t bufsize = 4;
-  char *buf = (char *)ares_malloc(bufsize);
-
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("abcde", std::string(buf));
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("0123456789", std::string(buf));
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("XYZ", std::string(buf));
-  SetAllocFail(1);
-  EXPECT_EQ(ARES_ENOMEM, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ(nullptr, buf);
-
-  fclose(fp);
-  ares_free(buf);
-}
-
-TEST_F(LibraryTest, ReadLineNoBuf) {
-  TempFile temp("abcde\n0123456789\nXYZ\n012345678901234567890");
-  FILE *fp = fopen(temp.filename(), "r");
-  size_t bufsize = 0;
-  char *buf = nullptr;
-
-  SetAllocFail(1);
-  EXPECT_EQ(ARES_ENOMEM, ares__read_line(fp, &buf, &bufsize));
-
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("abcde", std::string(buf));
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("0123456789", std::string(buf));
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("XYZ", std::string(buf));
-  EXPECT_EQ(ARES_SUCCESS, ares__read_line(fp, &buf, &bufsize));
-  EXPECT_EQ("012345678901234567890", std::string(buf));
-
-  fclose(fp);
-  ares_free(buf);
-}
-
-
 TEST_F(FileChannelTest, GetAddrInfoHostsPositive) {
   TempFile hostsfile("1.2.3.4 example.com  \n"
                      "  2.3.4.5\tgoogle.com   www.google.com\twww2.google.com\n"
@@ -446,6 +404,8 @@ TEST(Misc, OnionDomain) {
   EXPECT_EQ(1, ares__is_onion_domain("YES.ONION."));
 }
 
+#endif
+
 TEST_F(LibraryTest, DNSRecord) {
   ares_dns_record_t   *dnsrec = NULL;
   ares_dns_rr_t       *rr     = NULL;
@@ -474,14 +434,14 @@ TEST_F(LibraryTest, DNSRecord) {
   EXPECT_EQ(ARES_SUCCESS,
     ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER, "example.com",
       ARES_REC_TYPE_A, ARES_CLASS_IN, 300));
-  EXPECT_LT(0, ares_inet_net_pton(AF_INET, "1.1.1.1", &addr, sizeof(addr)));
+  EXPECT_LT(0, ares_inet_pton(AF_INET, "1.1.1.1", &addr));
   EXPECT_EQ(ARES_SUCCESS,
     ares_dns_rr_set_addr(rr, ARES_RR_A_ADDR, &addr));
   /* AAAA */
   EXPECT_EQ(ARES_SUCCESS,
     ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER, "example.com",
       ARES_REC_TYPE_AAAA, ARES_CLASS_IN, 300));
-  EXPECT_LT(0, ares_inet_net_pton(AF_INET6, "2600::4", &addr6, sizeof(addr6)));
+  EXPECT_LT(0, ares_inet_pton(AF_INET6, "2600::4", &addr6));
   EXPECT_EQ(ARES_SUCCESS,
     ares_dns_rr_set_addr6(rr, ARES_RR_AAAA_ADDR, &addr6));
   /* MX */
@@ -502,10 +462,41 @@ TEST_F(LibraryTest, DNSRecord) {
   EXPECT_EQ(ARES_SUCCESS,
     ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER, "example.com",
       ARES_REC_TYPE_TXT, ARES_CLASS_IN, 3600));
-  const char txt[] = "blah=here blah=there anywhere";
+  const char txt1[] = "blah=here blah=there anywhere";
+  const char txt2[] = "some other record";
   EXPECT_EQ(ARES_SUCCESS,
-    ares_dns_rr_set_bin(rr, ARES_RR_TXT_DATA, (unsigned char *)txt,
-      sizeof(txt)));
+    ares_dns_rr_add_abin(rr, ARES_RR_TXT_DATA, (unsigned char *)txt1,
+      sizeof(txt1)-1));
+   EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_add_abin(rr, ARES_RR_TXT_DATA, (unsigned char *)txt2,
+      sizeof(txt2)-1));
+  /* SIG */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER, "example.com",
+      ARES_REC_TYPE_SIG, ARES_CLASS_ANY, 0));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u16(rr, ARES_RR_SIG_TYPE_COVERED, ARES_REC_TYPE_TXT));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u8(rr, ARES_RR_SIG_ALGORITHM, 1));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u8(rr, ARES_RR_SIG_LABELS, 1));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u32(rr, ARES_RR_SIG_ORIGINAL_TTL, 3200));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u32(rr, ARES_RR_SIG_EXPIRATION, (unsigned int)time(NULL)));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u32(rr, ARES_RR_SIG_INCEPTION, (unsigned int)time(NULL) - (86400 * 365)));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u16(rr, ARES_RR_SIG_KEY_TAG, 0x1234));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_str(rr, ARES_RR_SIG_SIGNERS_NAME, "signer.example.com"));
+  const unsigned char sig[] = {
+    0xd2, 0xab, 0xde, 0x24, 0x0d, 0x7c, 0xd3, 0xee, 0x6b, 0x4b, 0x28, 0xc5,
+    0x4d, 0xf0, 0x34, 0xb9, 0x79, 0x83, 0xa1, 0xd1, 0x6e, 0x8a, 0x41, 0x0e,
+    0x45, 0x61, 0xcb, 0x10, 0x66, 0x18, 0xe9, 0x71 };
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_bin(rr, ARES_RR_SIG_SIGNATURE, sig, sizeof(sig)));
+
 
   /* == Authority == */
   /* NS */
@@ -686,16 +677,19 @@ TEST_F(LibraryTest, DNSRecord) {
   /* Write */
   EXPECT_EQ(ARES_SUCCESS, ares_dns_write(dnsrec, &msg, &msglen));
 
+#ifndef CARES_SYMBOL_HIDING
   ares__buf_t *hexdump = ares__buf_create();
   EXPECT_EQ(ARES_SUCCESS, ares__buf_hexdump(hexdump, msg, msglen));
   char *hexdata = ares__buf_finish_str(hexdump, NULL);
   //printf("HEXDUMP\n%s", hexdata);
   ares_free(hexdata);
+#endif
+
   ares_dns_record_destroy(dnsrec); dnsrec = NULL;
 
   /* Parse */
   EXPECT_EQ(ARES_SUCCESS, ares_dns_parse(msg, msglen, 0, &dnsrec));
-  ares_free(msg); msg = NULL;
+  ares_free_string(msg); msg = NULL;
 
   /* Re-write */
   EXPECT_EQ(ARES_SUCCESS, ares_dns_write(dnsrec, &msg, &msglen));
@@ -705,6 +699,7 @@ TEST_F(LibraryTest, DNSRecord) {
   EXPECT_EQ(nscount, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_AUTHORITY));
   EXPECT_EQ(arcount, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_ADDITIONAL));
 
+#ifndef CARES_SYMBOL_HIDING
   /* Iterate and print */
   ares__buf_t *printmsg = ares__buf_create();
   ares__buf_append_str(printmsg, ";; ->>HEADER<<- opcode: ");
@@ -744,7 +739,7 @@ TEST_F(LibraryTest, DNSRecord) {
     ares__buf_append_str(printmsg, ares_dns_section_tostr((ares_dns_section_t)i));
     ares__buf_append_str(printmsg, " SECTION:\n");
     for (size_t j = 0; j < ares_dns_record_rr_cnt(dnsrec, (ares_dns_section_t)i); j++) {
-      const ares_dns_rr_t *rr = ares_dns_record_rr_get(dnsrec, (ares_dns_section_t)i, j);
+      rr = ares_dns_record_rr_get(dnsrec, (ares_dns_section_t)i, j);
       ares__buf_append_str(printmsg, ares_dns_rr_get_name(rr));
       ares__buf_append_str(printmsg, ".\t\t\t");
       ares__buf_append_str(printmsg, ares_dns_class_tostr(ares_dns_rr_get_class(rr)));
@@ -793,6 +788,17 @@ TEST_F(LibraryTest, DNSRecord) {
             ares__buf_append_str(printmsg, (const char *)ares_dns_rr_get_bin(rr, keys[k], &templen));
             ares__buf_append_byte(printmsg, '"');
             break;
+          case ARES_DATATYPE_ABINP:
+            for (size_t a=0; a<ares_dns_rr_get_abin_cnt(rr, keys[k]); a++) {
+              if (a != 0) {
+                ares__buf_append_byte(printmsg, ' ');
+              }
+              ares__buf_append_byte(printmsg, '"');
+              size_t templen;
+              ares__buf_append_str(printmsg, (const char *)ares_dns_rr_get_abin(rr, keys[k], a, &templen));
+              ares__buf_append_byte(printmsg, '"');
+            }
+            break;
           case ARES_DATATYPE_OPT:
             /* TODO */
             break;
@@ -809,11 +815,174 @@ TEST_F(LibraryTest, DNSRecord) {
   char *printdata = ares__buf_finish_str(printmsg, NULL);
   //printf("%s", printdata);
   ares_free(printdata);
+#endif
 
   ares_dns_record_destroy(dnsrec);
-  ares_free(msg);
+  ares_free_string(msg);
+
+  // Invalid
+  EXPECT_NE(ARES_SUCCESS, ares_dns_parse(NULL, 0, 0, NULL));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_create(NULL, 0, 0, ARES_OPCODE_QUERY, ARES_RCODE_NOERROR));
+  EXPECT_EQ(0, ares_dns_record_get_id(NULL));
+  EXPECT_EQ(0, ares_dns_record_get_flags(NULL));
+  EXPECT_EQ(0, (int)ares_dns_record_get_opcode(NULL));
+  EXPECT_EQ(0, (int)ares_dns_record_get_rcode(NULL));
+  EXPECT_EQ(0, (int)ares_dns_record_query_cnt(NULL));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_query_set_name(NULL, 0, NULL));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_query_set_type(NULL, 0, ARES_REC_TYPE_A));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_query_get(NULL, 0, NULL, NULL, NULL));
+  EXPECT_EQ(0, ares_dns_record_rr_cnt(NULL, ARES_SECTION_ANSWER));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_rr_add(NULL, NULL, ARES_SECTION_ANSWER, NULL, ARES_REC_TYPE_A, ARES_CLASS_IN, 0));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_record_rr_del(NULL, ARES_SECTION_ANSWER, 0));
+  EXPECT_EQ(nullptr, ares_dns_record_rr_get(NULL, ARES_SECTION_ANSWER, 0));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_name(NULL));
+  EXPECT_EQ(0, (int)ares_dns_rr_get_type(NULL));
+  EXPECT_EQ(0, (int)ares_dns_rr_get_class(NULL));
+  EXPECT_EQ(0, ares_dns_rr_get_ttl(NULL));
+  EXPECT_NE(ARES_SUCCESS, ares_dns_write(NULL, NULL, NULL));
+#ifndef CARES_SYMBOL_HIDING
+  ares_dns_record_write_ttl_decrement(NULL, 0);
+#endif
+  EXPECT_EQ(nullptr, ares_dns_rr_get_addr(NULL, ARES_RR_A_ADDR));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_addr(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_addr6(NULL, ARES_RR_AAAA_ADDR));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_addr6(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(0, ares_dns_rr_get_u8(NULL, ARES_RR_SIG_ALGORITHM));
+  EXPECT_EQ(0, ares_dns_rr_get_u8(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(0, ares_dns_rr_get_u16(NULL, ARES_RR_MX_PREFERENCE));
+  EXPECT_EQ(0, ares_dns_rr_get_u16(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(0, ares_dns_rr_get_u32(NULL, ARES_RR_SOA_SERIAL));
+  EXPECT_EQ(0, ares_dns_rr_get_u32(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_bin(NULL, ARES_RR_TXT_DATA, NULL));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_bin(NULL, ARES_RR_NS_NSDNAME, NULL));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_str(NULL, ARES_RR_NS_NSDNAME));
+  EXPECT_EQ(nullptr, ares_dns_rr_get_str(NULL, ARES_RR_MX_PREFERENCE));
+  EXPECT_EQ(0, ares_dns_rr_get_opt_cnt(NULL, ARES_RR_OPT_OPTIONS));
+  EXPECT_EQ(0, ares_dns_rr_get_opt_cnt(NULL, ARES_RR_A_ADDR));
+  EXPECT_EQ(65535, ares_dns_rr_get_opt(NULL, ARES_RR_OPT_OPTIONS, 0, NULL, NULL));
+  EXPECT_EQ(65535, ares_dns_rr_get_opt(NULL, ARES_RR_A_ADDR, 0, NULL, NULL));
+  EXPECT_EQ(ARES_FALSE, ares_dns_rr_get_opt_byid(NULL, ARES_RR_OPT_OPTIONS, 1, NULL, NULL));
+  EXPECT_EQ(ARES_FALSE, ares_dns_rr_get_opt_byid(NULL, ARES_RR_A_ADDR, 1, NULL, NULL));
 }
 
+TEST_F(LibraryTest, DNSParseFlags) {
+  ares_dns_record_t   *dnsrec = NULL;
+  ares_dns_rr_t       *rr     = NULL;
+  struct in_addr       addr;
+  unsigned char       *msg    = NULL;
+  size_t               msglen = 0;
+
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_create(&dnsrec, 0x1234,
+      ARES_FLAG_QR|ARES_FLAG_AA|ARES_FLAG_RD|ARES_FLAG_RA,
+      ARES_OPCODE_QUERY, ARES_RCODE_NOERROR));
+
+  /* == Question == */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_query_add(dnsrec, "example.com",
+      ARES_REC_TYPE_ANY,
+      ARES_CLASS_IN));
+
+  /* == Answer == */
+  /* A */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER, "example.com",
+      ARES_REC_TYPE_A, ARES_CLASS_IN, 300));
+  EXPECT_LT(0, ares_inet_pton(AF_INET, "1.1.1.1", &addr));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_addr(rr, ARES_RR_A_ADDR, &addr));
+  /* TLSA */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ANSWER,
+      "_443._tcp.example.com", ARES_REC_TYPE_TLSA, ARES_CLASS_IN, 86400));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u8(rr, ARES_RR_TLSA_CERT_USAGE, ARES_TLSA_USAGE_CA));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u8(rr, ARES_RR_TLSA_SELECTOR, ARES_TLSA_SELECTOR_FULL));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_u8(rr, ARES_RR_TLSA_MATCH, ARES_TLSA_MATCH_SHA256));
+  const unsigned char tlsa[] = {
+    0xd2, 0xab, 0xde, 0x24, 0x0d, 0x7c, 0xd3, 0xee, 0x6b, 0x4b, 0x28, 0xc5,
+    0x4d, 0xf0, 0x34, 0xb9, 0x79, 0x83, 0xa1, 0xd1, 0x6e, 0x8a, 0x41, 0x0e,
+    0x45, 0x61, 0xcb, 0x10, 0x66, 0x18, 0xe9, 0x71 };
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_bin(rr, ARES_RR_TLSA_DATA, tlsa, sizeof(tlsa)));
+
+  /* == Authority == */
+  /* NS */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_AUTHORITY, "example.com",
+      ARES_REC_TYPE_NS, ARES_CLASS_IN, 38400));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_str(rr, ARES_RR_NS_NSDNAME, "ns1.example.com"));
+
+  /* == Additional */
+  /* PTR -- doesn't make sense, but ok */
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_record_rr_add(&rr, dnsrec, ARES_SECTION_ADDITIONAL, "example.com",
+      ARES_REC_TYPE_PTR, ARES_CLASS_IN, 300));
+  EXPECT_EQ(ARES_SUCCESS,
+    ares_dns_rr_set_str(rr, ARES_RR_PTR_DNAME, "b.example.com"));
+
+  /* Write */
+  EXPECT_EQ(ARES_SUCCESS, ares_dns_write(dnsrec, &msg, &msglen));
+
+  /* Cleanup - before reuse */
+  ares_dns_record_destroy(dnsrec);
+
+  /* Parse "base" type records (1035) */
+  EXPECT_EQ(ARES_SUCCESS, ares_dns_parse(msg, msglen, ARES_DNS_PARSE_AN_BASE_RAW |
+    ARES_DNS_PARSE_NS_BASE_RAW | ARES_DNS_PARSE_AR_BASE_RAW, &dnsrec));
+
+  EXPECT_EQ(1, ares_dns_record_query_cnt(dnsrec));
+  EXPECT_EQ(2, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_ANSWER));
+  EXPECT_EQ(1, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_AUTHORITY));
+  EXPECT_EQ(1, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_ADDITIONAL));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ANSWER, 0);
+  EXPECT_EQ(ARES_REC_TYPE_RAW_RR, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ANSWER, 1);
+  EXPECT_EQ(ARES_REC_TYPE_TLSA, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_AUTHORITY, 0);
+  EXPECT_EQ(ARES_REC_TYPE_RAW_RR, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ADDITIONAL, 0);
+  EXPECT_EQ(ARES_REC_TYPE_RAW_RR, ares_dns_rr_get_type(rr));
+
+  /* Cleanup - before reuse */
+
+  ares_dns_record_destroy(dnsrec);
+
+  /* Parse later RFCs (no name compression) type records */
+
+  EXPECT_EQ(ARES_SUCCESS, ares_dns_parse(msg, msglen, ARES_DNS_PARSE_AN_EXT_RAW |
+    ARES_DNS_PARSE_NS_EXT_RAW | ARES_DNS_PARSE_AR_EXT_RAW, &dnsrec));
+
+  EXPECT_EQ(1, ares_dns_record_query_cnt(dnsrec));
+  EXPECT_EQ(2, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_ANSWER));
+  EXPECT_EQ(1, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_AUTHORITY));
+  EXPECT_EQ(1, ares_dns_record_rr_cnt(dnsrec, ARES_SECTION_ADDITIONAL));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ANSWER, 0);
+  EXPECT_EQ(ARES_REC_TYPE_A, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ANSWER, 1);
+  EXPECT_EQ(ARES_REC_TYPE_RAW_RR, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_AUTHORITY, 0);
+  EXPECT_EQ(ARES_REC_TYPE_NS, ares_dns_rr_get_type(rr));
+
+  rr = ares_dns_record_rr_get(dnsrec, ARES_SECTION_ADDITIONAL, 0);
+  EXPECT_EQ(ARES_REC_TYPE_PTR, ares_dns_rr_get_type(rr));
+
+  ares_dns_record_destroy(dnsrec);
+  ares_free_string(msg); msg = NULL;
+}
+
+#ifndef CARES_SYMBOL_HIDING
+
 TEST_F(LibraryTest, CatDomain) {
   char *s;
 
@@ -856,7 +1025,7 @@ TEST_F(LibraryTest, BufMisuse) {
   EXPECT_EQ((size_t)0, ares__buf_get_position(NULL));
   EXPECT_NE(ARES_SUCCESS, ares__buf_set_position(NULL, 0));
   EXPECT_NE(ARES_SUCCESS, ares__dns_name_parse(NULL, NULL, ARES_FALSE));
-  EXPECT_NE(ARES_SUCCESS, ares__buf_parse_dns_binstr(NULL, 0, NULL, NULL, ARES_FALSE));
+  EXPECT_NE(ARES_SUCCESS, ares__buf_parse_dns_binstr(NULL, 0, NULL, NULL));
 }
 
 TEST_F(LibraryTest, HtableMisuse) {
@@ -888,6 +1057,13 @@ TEST_F(LibraryTest, HtableSzvpMisuse) {
   EXPECT_EQ((size_t)0, ares__htable_szvp_num_keys(NULL));
 }
 
+TEST_F(LibraryTest, HtableVpvpMisuse) {
+  EXPECT_EQ(ARES_FALSE, ares__htable_vpvp_insert(NULL, NULL, NULL));
+  EXPECT_EQ(ARES_FALSE, ares__htable_vpvp_get(NULL, NULL, NULL));
+  EXPECT_EQ(ARES_FALSE, ares__htable_vpvp_remove(NULL, NULL));
+  EXPECT_EQ((size_t)0, ares__htable_vpvp_num_keys(NULL));
+}
+
 TEST_F(LibraryTest, LlistMisuse) {
   ares__llist_replace_destructor(NULL, NULL);
   EXPECT_EQ(NULL, ares__llist_insert_before(NULL, NULL));
@@ -917,42 +1093,257 @@ TEST_F(LibraryTest, SlistMisuse) {
   EXPECT_EQ(NULL, ares__slist_last_val(NULL));
   EXPECT_EQ(NULL, ares__slist_node_claim(NULL));
 }
-#endif
 
-#ifdef CARES_EXPOSE_STATICS
-// These tests access internal static functions from the library, which
-// are only exposed when CARES_EXPOSE_STATICS has been configured. As such
-// they are tightly couple to the internal library implementation details.
-extern "C" char *ares_striendstr(const char*, const char*);
-TEST_F(LibraryTest, Striendstr) {
-  EXPECT_EQ(nullptr, ares_striendstr("abc", "12345"));
-  EXPECT_NE(nullptr, ares_striendstr("abc12345", "12345"));
-  EXPECT_NE(nullptr, ares_striendstr("abcxyzzy", "XYZZY"));
-  EXPECT_NE(nullptr, ares_striendstr("xyzzy", "XYZZY"));
-  EXPECT_EQ(nullptr, ares_striendstr("xyxzy", "XYZZY"));
-  EXPECT_NE(nullptr, ares_striendstr("", ""));
-  const char *str = "plugh";
-  EXPECT_NE(nullptr, ares_striendstr(str, str));
+
+TEST_F(LibraryTest, HtableVpvp) {
+  ares__llist_t       *l = NULL;
+  ares__htable_vpvp_t *h = NULL;
+  ares__llist_node_t  *n = NULL;
+  size_t               i;
+
+#define VPVP_TABLE_SIZE 1000
+
+  l = ares__llist_create(NULL);
+  EXPECT_NE((void *)NULL, l);
+
+  h = ares__htable_vpvp_create(NULL, ares_free);
+  EXPECT_NE((void *)NULL, h);
+
+  for (i=0; i<VPVP_TABLE_SIZE; i++) {
+    void *p = ares_malloc_zero(4);
+    EXPECT_NE((void *)NULL, p);
+    EXPECT_NE((void *)NULL, ares__llist_insert_last(l, p));
+    EXPECT_TRUE(ares__htable_vpvp_insert(h, p, p));
+  }
+
+  EXPECT_EQ(VPVP_TABLE_SIZE, ares__llist_len(l));
+  EXPECT_EQ(VPVP_TABLE_SIZE, ares__htable_vpvp_num_keys(h));
+
+  n = ares__llist_node_first(l);
+  EXPECT_NE((void *)NULL, n);
+  while (n != NULL) {
+    ares__llist_node_t *next = ares__llist_node_next(n);
+    void               *p    = ares__llist_node_val(n);
+    EXPECT_NE((void *)NULL, p);
+    EXPECT_EQ(p, ares__htable_vpvp_get_direct(h, p));
+    EXPECT_TRUE(ares__htable_vpvp_get(h, p, NULL));
+    EXPECT_TRUE(ares__htable_vpvp_remove(h, p));
+    ares__llist_node_destroy(n);
+    n = next;
+  }
+
+  EXPECT_EQ(0, ares__llist_len(l));
+  EXPECT_EQ(0, ares__htable_vpvp_num_keys(h));
+
+  ares__llist_destroy(l);
+  ares__htable_vpvp_destroy(h);
 }
 
-TEST_F(DefaultChannelTest, SingleDomain) {
-  TempFile aliases("www www.google.com\n");
-  EnvValue with_env("HOSTALIASES", aliases.filename());
+typedef struct {
+  ares_socket_t s;
+} test_htable_asvp_t;
 
-  SetAllocSizeFail(128);
-  char *ptr = nullptr;
-  EXPECT_EQ(ARES_ENOMEM, ares__single_domain(channel_, "www", &ptr));
+TEST_F(LibraryTest, HtableAsvp) {
+  ares__llist_t       *l = NULL;
+  ares__htable_asvp_t *h = NULL;
+  ares__llist_node_t  *n = NULL;
+  size_t               i;
 
-  channel_->flags |= ARES_FLAG_NOSEARCH|ARES_FLAG_NOALIASES;
-  EXPECT_EQ(ARES_SUCCESS, ares__single_domain(channel_, "www", &ptr));
-  EXPECT_EQ("www", std::string(ptr));
-  ares_free(ptr);
-  ptr = nullptr;
+#define ASVP_TABLE_SIZE 1000
 
-  SetAllocFail(1);
-  EXPECT_EQ(ARES_ENOMEM, ares__single_domain(channel_, "www", &ptr));
-  EXPECT_EQ(nullptr, ptr);
+  l = ares__llist_create(NULL);
+  EXPECT_NE((void *)NULL, l);
+
+  h = ares__htable_asvp_create(ares_free);
+  EXPECT_NE((void *)NULL, h);
+
+  for (i=0; i<ASVP_TABLE_SIZE; i++) {
+    test_htable_asvp_t *a = (test_htable_asvp_t *)ares_malloc_zero(sizeof(*a));
+    EXPECT_NE((void *)NULL, a);
+    a->s = (ares_socket_t)i+1;
+    EXPECT_NE((void *)NULL, ares__llist_insert_last(l, a));
+    EXPECT_TRUE(ares__htable_asvp_insert(h, a->s, a));
+  }
+
+  EXPECT_EQ(ASVP_TABLE_SIZE, ares__llist_len(l));
+  EXPECT_EQ(ASVP_TABLE_SIZE, ares__htable_asvp_num_keys(h));
+
+  n = ares__llist_node_first(l);
+  EXPECT_NE((void *)NULL, n);
+  while (n != NULL) {
+    ares__llist_node_t *next = ares__llist_node_next(n);
+    test_htable_asvp_t *a    = (test_htable_asvp_t *)ares__llist_node_val(n);
+    EXPECT_NE((void *)NULL, a);
+    EXPECT_EQ(a, ares__htable_asvp_get_direct(h, a->s));
+    EXPECT_TRUE(ares__htable_asvp_get(h, a->s, NULL));
+    EXPECT_TRUE(ares__htable_asvp_remove(h, a->s));
+    ares__llist_node_destroy(n);
+    n = next;
+  }
+
+  EXPECT_EQ(0, ares__llist_len(l));
+  EXPECT_EQ(0, ares__htable_asvp_num_keys(h));
+
+  ares__llist_destroy(l);
+  ares__htable_asvp_destroy(h);
 }
+
+
+typedef struct {
+  size_t s;
+} test_htable_szvp_t;
+
+TEST_F(LibraryTest, HtableSzvp) {
+  ares__llist_t       *l = NULL;
+  ares__htable_szvp_t *h = NULL;
+  ares__llist_node_t  *n = NULL;
+  size_t               i;
+
+#define SZVP_TABLE_SIZE 1000
+
+  l = ares__llist_create(NULL);
+  EXPECT_NE((void *)NULL, l);
+
+  h = ares__htable_szvp_create(ares_free);
+  EXPECT_NE((void *)NULL, h);
+
+  for (i=0; i<SZVP_TABLE_SIZE; i++) {
+    test_htable_szvp_t *s = (test_htable_szvp_t *)ares_malloc_zero(sizeof(*s));
+    EXPECT_NE((void *)NULL, s);
+    s->s = i+1;
+    EXPECT_NE((void *)NULL, ares__llist_insert_last(l, s));
+    EXPECT_TRUE(ares__htable_szvp_insert(h, s->s, s));
+  }
+
+  EXPECT_EQ(SZVP_TABLE_SIZE, ares__llist_len(l));
+  EXPECT_EQ(SZVP_TABLE_SIZE, ares__htable_szvp_num_keys(h));
+
+  n = ares__llist_node_first(l);
+  EXPECT_NE((void *)NULL, n);
+  while (n != NULL) {
+    ares__llist_node_t *next = ares__llist_node_next(n);
+    test_htable_szvp_t *s    = (test_htable_szvp_t *)ares__llist_node_val(n);
+    EXPECT_NE((void *)NULL, s);
+    EXPECT_EQ(s, ares__htable_szvp_get_direct(h, s->s));
+    EXPECT_TRUE(ares__htable_szvp_get(h, s->s, NULL));
+    EXPECT_TRUE(ares__htable_szvp_remove(h, s->s));
+    ares__llist_node_destroy(n);
+    n = next;
+  }
+
+  EXPECT_EQ(0, ares__llist_len(l));
+  EXPECT_EQ(0, ares__htable_szvp_num_keys(h));
+
+  ares__llist_destroy(l);
+  ares__htable_szvp_destroy(h);
+}
+
+typedef struct {
+  char s[32];
+} test_htable_strvp_t;
+
+TEST_F(LibraryTest, HtableStrvp) {
+  ares__llist_t        *l = NULL;
+  ares__htable_strvp_t *h = NULL;
+  ares__llist_node_t   *n = NULL;
+  size_t                i;
+
+#define STRVP_TABLE_SIZE 1000
+
+  l = ares__llist_create(NULL);
+  EXPECT_NE((void *)NULL, l);
+
+  h = ares__htable_strvp_create(ares_free);
+  EXPECT_NE((void *)NULL, h);
+
+  for (i=0; i<STRVP_TABLE_SIZE; i++) {
+    test_htable_strvp_t *s = (test_htable_strvp_t *)ares_malloc_zero(sizeof(*s));
+    EXPECT_NE((void *)NULL, s);
+    snprintf(s->s, sizeof(s->s), "%d", (int)i);
+    EXPECT_NE((void *)NULL, ares__llist_insert_last(l, s));
+    EXPECT_TRUE(ares__htable_strvp_insert(h, s->s, s));
+  }
+
+  EXPECT_EQ(STRVP_TABLE_SIZE, ares__llist_len(l));
+  EXPECT_EQ(STRVP_TABLE_SIZE, ares__htable_strvp_num_keys(h));
+
+  n = ares__llist_node_first(l);
+  EXPECT_NE((void *)NULL, n);
+  while (n != NULL) {
+    ares__llist_node_t *next = ares__llist_node_next(n);
+    test_htable_strvp_t *s   = (test_htable_strvp_t *)ares__llist_node_val(n);
+    EXPECT_NE((void *)NULL, s);
+    EXPECT_EQ(s, ares__htable_strvp_get_direct(h, s->s));
+    EXPECT_TRUE(ares__htable_strvp_get(h, s->s, NULL));
+    EXPECT_TRUE(ares__htable_strvp_remove(h, s->s));
+    ares__llist_node_destroy(n);
+    n = next;
+  }
+
+  EXPECT_EQ(0, ares__llist_len(l));
+  EXPECT_EQ(0, ares__htable_strvp_num_keys(h));
+
+  ares__llist_destroy(l);
+  ares__htable_strvp_destroy(h);
+}
+
+TEST_F(LibraryTest, IfaceIPs) {
+  ares_status_t      status;
+  ares__iface_ips_t *ips = NULL;
+  size_t             i;
+
+  status = ares__iface_ips(&ips, ARES_IFACE_IP_DEFAULT, NULL);
+  EXPECT_TRUE(status == ARES_SUCCESS || status == ARES_ENOTIMP);
+
+  /* Not implemented, can't run tests */
+  if (status == ARES_ENOTIMP)
+    return;
+
+  EXPECT_NE(nullptr, ips);
+
+  for (i=0; i<ares__iface_ips_cnt(ips); i++) {
+    const char *name = ares__iface_ips_get_name(ips, i);
+    EXPECT_NE(nullptr, name);
+    int flags = (int)ares__iface_ips_get_flags(ips, i);
+    EXPECT_NE(0, (int)flags);
+    EXPECT_NE(nullptr, ares__iface_ips_get_addr(ips, i));
+    EXPECT_NE(0, ares__iface_ips_get_netmask(ips, i));
+    if (flags & ARES_IFACE_IP_LINKLOCAL && flags & ARES_IFACE_IP_V6) {
+      /* Hmm, seems not to work at least on MacOS
+       * EXPECT_NE(0, ares__iface_ips_get_ll_scope(ips, i));
+       */
+    } else {
+      EXPECT_EQ(0, ares__iface_ips_get_ll_scope(ips, i));
+    }
+    unsigned int idx = ares__if_nametoindex(name);
+    EXPECT_NE(0, idx);
+    char namebuf[256];
+    EXPECT_EQ(std::string(ares__if_indextoname(idx, namebuf, sizeof(namebuf))), std::string(name));
+  }
+
+
+  /* Negative checking */
+  ares__iface_ips_get_name(ips, ares__iface_ips_cnt(ips));
+  ares__iface_ips_get_flags(ips, ares__iface_ips_cnt(ips));
+  ares__iface_ips_get_addr(ips, ares__iface_ips_cnt(ips));
+  ares__iface_ips_get_netmask(ips, ares__iface_ips_cnt(ips));
+  ares__iface_ips_get_ll_scope(ips, ares__iface_ips_cnt(ips));
+
+  ares__iface_ips(NULL, ARES_IFACE_IP_DEFAULT, NULL);
+  ares__iface_ips_cnt(NULL);
+  ares__iface_ips_get_name(NULL, 0);
+  ares__iface_ips_get_flags(NULL, 0);
+  ares__iface_ips_get_addr(NULL, 0);
+  ares__iface_ips_get_netmask(NULL, 0);
+  ares__iface_ips_get_ll_scope(NULL, 0);
+  ares__iface_ips_destroy(NULL);
+  ares__if_nametoindex(NULL);
+  ares__if_indextoname(0, NULL, 0);
+
+  ares__iface_ips_destroy(ips);
+}
+
+
 #endif
 
 TEST_F(DefaultChannelTest, SaveInvalidChannel) {
