@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,6 +26,8 @@
 #include <fluent-bit/flb_mem.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_kv.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 #include <msgpack.h>
 
 #include <string.h>
@@ -97,7 +98,6 @@ static int configure(struct filter_parser_ctx *ctx,
                      struct flb_config *config)
 {
     int ret;
-    const char *tmp;
     struct mk_list *head;
     struct flb_kv *kv;
 
@@ -106,16 +106,17 @@ static int configure(struct filter_parser_ctx *ctx,
     ctx->preserve_key = FLB_FALSE;
     mk_list_init(&ctx->parsers);
 
-    /* Key name */
-    tmp = flb_filter_get_property("key_name", f_ins);
-    if (tmp) {
-        ctx->key_name = flb_strdup(tmp);
-        ctx->key_name_len = strlen(tmp);
+    if (flb_filter_config_map_set(f_ins, ctx) < 0) {
+        flb_errno();
+        flb_plg_error(f_ins, "configuration error");
+        return -1;
     }
-    else {
+
+    if (ctx->key_name == NULL) {
         flb_plg_error(ctx->ins, "missing 'key_name'");
         return -1;
     }
+    ctx->key_name_len = flb_sds_len(ctx->key_name);
 
     /* Read all Parsers */
     mk_list_foreach(head, &f_ins->properties) {
@@ -123,7 +124,6 @@ static int configure(struct filter_parser_ctx *ctx,
         if (strcasecmp("parser", kv->key) != 0) {
             continue;
         }
-
         ret = add_parser(kv->val, ctx, config);
         if (ret == -1) {
             flb_plg_error(ctx->ins, "requested parser '%s' not found", kv->val);
@@ -135,18 +135,6 @@ static int configure(struct filter_parser_ctx *ctx,
         return -1;
     }
 
-    /* Reserve data */
-    tmp = flb_filter_get_property("reserve_data", f_ins);
-    if (tmp) {
-        ctx->reserve_data = flb_utils_bool(tmp);
-    }
-
-    /* Preserve key */
-    tmp = flb_filter_get_property("preserve_key", f_ins);
-    if (tmp) {
-        ctx->preserve_key = flb_utils_bool(tmp);
-    }
-
     return 0;
 }
 
@@ -188,11 +176,6 @@ static int cb_parser_filter(const void *data, size_t bytes,
 {
     int continue_parsing;
     struct filter_parser_ctx *ctx = context;
-    msgpack_unpacked result;
-    size_t off = 0;
-    (void) f_ins;
-    (void) i_ins;
-    (void) config;
     struct flb_time tm;
     msgpack_object *obj;
 
@@ -208,42 +191,64 @@ static int cb_parser_filter(const void *data, size_t bytes,
     char *out_buf;
     size_t out_size;
     struct flb_time parsed_time;
-    msgpack_sbuffer tmp_sbuf;
-    msgpack_packer tmp_pck;
 
     msgpack_object_kv **append_arr = NULL;
-    size_t            append_arr_len = 0;
-    int                append_arr_i;
+    size_t append_arr_len = 0;
+    int append_arr_i;
     struct mk_list *head;
     struct filter_parser *fp;
+    struct flb_log_event_encoder log_encoder;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    int encoder_result;
 
-    /* Create temporary msgpack buffer */
-    msgpack_sbuffer_init(&tmp_sbuf);
-    msgpack_packer_init(&tmp_pck, &tmp_sbuf, msgpack_sbuffer_write);
+    (void) f_ins;
+    (void) i_ins;
+    (void) config;
 
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
+
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    ret = flb_log_event_encoder_init(&log_encoder,
+                                     FLB_LOG_EVENT_FORMAT_DEFAULT);
+
+    if (ret != FLB_EVENT_ENCODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder initialization error : %d", ret);
+
+        flb_log_event_decoder_destroy(&log_decoder);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
         out_buf = NULL;
         append_arr_i = 0;
 
-        if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
-        flb_time_pop_from_msgpack(&tm, &result, &obj);
+        flb_time_copy(&tm, &log_event.timestamp);
+        obj = log_event.body;
+
         if (obj->type == MSGPACK_OBJECT_MAP) {
             map_num = obj->via.map.size;
             if (ctx->reserve_data) {
                 append_arr_len = obj->via.map.size;
-                append_arr = flb_malloc(sizeof(msgpack_object_kv*) * append_arr_len);
-                if (!append_arr) {
-                    flb_errno();
-                    msgpack_unpacked_destroy(&result);
-                    msgpack_sbuffer_destroy(&tmp_sbuf);
-                    return FLB_FILTER_NOTOUCH;
-                }
+                append_arr = flb_calloc(append_arr_len, sizeof(msgpack_object_kv *));
 
-                for (i = 0; i < append_arr_len; i++){
-                    append_arr[i] = NULL;
+                if (append_arr == NULL) {
+                    flb_errno();
+
+                    flb_log_event_decoder_destroy(&log_decoder);
+                    flb_log_event_encoder_destroy(&log_encoder);
+
+                    return FLB_FILTER_NOTOUCH;
                 }
             }
 
@@ -284,7 +289,7 @@ static int cb_parser_filter(const void *data, size_t bytes,
                              * holder with the new value, otherwise keep the
                              * original.
                              */
-                            if (flb_time_to_double(&parsed_time) != 0.0) {
+                            if (flb_time_to_nanosec(&parsed_time) != 0L) {
                                 flb_time_copy(&tm, &parsed_time);
                             }
 
@@ -304,9 +309,20 @@ static int cb_parser_filter(const void *data, size_t bytes,
                 }
             }
 
+            encoder_result = flb_log_event_encoder_begin_record(&log_encoder);
+
+            if (encoder_result == FLB_EVENT_ENCODER_SUCCESS) {
+                encoder_result = flb_log_event_encoder_set_timestamp(
+                                     &log_encoder, &tm);
+            }
+
+            if (encoder_result == FLB_EVENT_ENCODER_SUCCESS) {
+                encoder_result = \
+                    flb_log_event_encoder_set_metadata_from_msgpack_object(
+                        &log_encoder, log_event.metadata);
+            }
+
             if (out_buf != NULL) {
-                msgpack_pack_array(&tmp_pck, 2);
-                flb_time_append_to_msgpack(&tm, &tmp_pck, 0);
                 if (ctx->reserve_data) {
                     char *new_buf = NULL;
                     int  new_size;
@@ -316,8 +332,11 @@ static int cb_parser_filter(const void *data, size_t bytes,
                                                  &new_buf, &new_size);
                     if (ret == -1) {
                         flb_plg_error(ctx->ins, "cannot expand map");
+
+                        flb_log_event_decoder_destroy(&log_decoder);
+                        flb_log_event_encoder_destroy(&log_encoder);
                         flb_free(append_arr);
-                        msgpack_unpacked_destroy(&result);
+
                         return FLB_FILTER_NOTOUCH;
                     }
 
@@ -325,14 +344,29 @@ static int cb_parser_filter(const void *data, size_t bytes,
                     out_buf = new_buf;
                     out_size = new_size;
                 }
-                msgpack_sbuffer_write(&tmp_sbuf, out_buf, out_size);
+
+                if (encoder_result == FLB_EVENT_ENCODER_SUCCESS) {
+                    encoder_result = \
+                        flb_log_event_encoder_set_body_from_raw_msgpack(
+                            &log_encoder, out_buf, out_size);
+                }
+
                 flb_free(out_buf);
                 ret = FLB_FILTER_MODIFIED;
             }
             else {
                 /* re-use original data*/
-                msgpack_pack_object(&tmp_pck, result.data);
+                if (encoder_result == FLB_EVENT_ENCODER_SUCCESS) {
+                    encoder_result = \
+                        flb_log_event_encoder_set_body_from_msgpack_object(
+                            &log_encoder, log_event.body);
+                }
             }
+
+            if (encoder_result == FLB_EVENT_ENCODER_SUCCESS) {
+                encoder_result = flb_log_event_encoder_commit_record(&log_encoder);
+            }
+
             flb_free(append_arr);
             append_arr = NULL;
         }
@@ -340,16 +374,24 @@ static int cb_parser_filter(const void *data, size_t bytes,
             continue;
         }
     }
-    msgpack_unpacked_destroy(&result);
 
-    if (ret == FLB_FILTER_NOTOUCH) {
-        /* Destroy the buffer to avoid more overhead */
-        msgpack_sbuffer_destroy(&tmp_sbuf);
-        return FLB_FILTER_NOTOUCH;
+    if (log_encoder.output_length > 0) {
+        *ret_buf   = log_encoder.output_buffer;
+        *ret_bytes = log_encoder.output_length;
+
+        ret = FLB_FILTER_MODIFIED;
+
+        flb_log_event_encoder_claim_internal_buffer_ownership(&log_encoder);
+    }
+    else {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder error : %d", ret);
+
+        ret = FLB_FILTER_NOTOUCH;
     }
 
-    *ret_buf = tmp_sbuf.data;
-    *ret_bytes = tmp_sbuf.size;
+    flb_log_event_decoder_destroy(&log_decoder);
+    flb_log_event_encoder_destroy(&log_encoder);
 
     return ret;
 }
@@ -364,16 +406,47 @@ static int cb_parser_exit(void *data, struct flb_config *config)
     }
 
     delete_parsers(ctx);
-    flb_free(ctx->key_name);
     flb_free(ctx);
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "Key_Name", NULL,
+     0, FLB_TRUE, offsetof(struct filter_parser_ctx, key_name),
+     "Specify field name in record to parse."
+    },
+    {
+     FLB_CONFIG_MAP_STR, "Parser", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_FALSE, 0,
+     "Specify the parser name to interpret the field. "
+     "Multiple Parser entries are allowed (one per line)."
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "Preserve_Key", "false",
+     0, FLB_TRUE, offsetof(struct filter_parser_ctx, preserve_key),
+     "Keep original Key_Name field in the parsed result. If false, the field will be removed."
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "Reserve_Data", "false",
+     0, FLB_TRUE, offsetof(struct filter_parser_ctx, reserve_data),
+     "Keep all other original fields in the parsed result. "
+     "If false, all other original fields will be removed."
+    },
+    {
+     FLB_CONFIG_MAP_DEPRECATED, "Unescape_key", NULL,
+     0, FLB_FALSE, 0,
+     "(deprecated)"
+    },
+    {0}
+};
+
 struct flb_filter_plugin filter_parser_plugin = {
     .name         = "parser",
     .description  = "Parse events",
     .cb_init      = cb_parser_init,
     .cb_filter    = cb_parser_filter,
     .cb_exit      = cb_parser_exit,
+    .config_map   = config_map,
     .flags        = 0
 };
