@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -24,10 +23,11 @@
 #include <fluent-bit/flb_config.h>
 #include <fluent-bit/flb_input.h>
 #include <fluent-bit/flb_utils.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
 /* Default collection time: every 1 second (0 nanoseconds) */
-#define DEFAULT_INTERVAL_SEC    1
-#define DEFAULT_INTERVAL_NSEC   0
+#define DEFAULT_INTERVAL_SEC    "1"
+#define DEFAULT_INTERVAL_NSEC   "0"
 #define IN_CPU_KEY_LEN       16
 
 struct cpu_key {
@@ -77,19 +77,14 @@ struct flb_cpu {
     int interval_nsec;  /* interval collection time (Nanosecond) */
     struct cpu_stats cstats;
     struct flb_input_instance *ins;
+    struct flb_log_event_encoder log_encoder;
 };
 
-
 #define CPU_KEY_FORMAT(s, key, i)                                   \
     s->k_##key.length = snprintf(s->k_##key.name,                   \
                                  IN_CPU_KEY_LEN,                    \
                                  "cpu%i.p_%s", i - 1, #key)
 
-#define CPU_PACK_SNAP(s, key)                                           \
-    msgpack_pack_str(&mp_pck, s->k_##key.length);                       \
-    msgpack_pack_str_body(&mp_pck, s->k_##key.name, s->k_##key.length); \
-    msgpack_pack_double(&mp_pck, s->p_##key)
-
 #define ULL_ABS(a, b)  (a > b) ? a - b : b - a
 
 /*
