@@ -20,6 +20,8 @@
 #ifndef FLB_IN_ELASTICSEARCH_BULK_PROT
 #define FLB_IN_ELASTICSEARCH_BULK_PROT
 
+#include "in_elasticsearch_bulk_conn.h"
+
 #define ES_VERSION_RESPONSE_TEMPLATE \
     "{\"version\":{\"number\":\"%s\",\"build_flavor\":\"Fluent Bit OSS\"},\"tagline\":\"Fluent Bit's Bulk API compatible endpoint\"}"
 
@@ -37,4 +39,8 @@ int in_elasticsearch_bulk_prot_handle_error(struct flb_in_elasticsearch *ctx,
                                             struct mk_http_session *session,
                                             struct mk_http_request *request);
 
+
+int in_elasticsearch_bulk_prot_handle_ng(struct flb_http_request *request,
+                                         struct flb_http_response *response);
+
 #endif
