@@ -630,6 +630,7 @@ static int check_ping(struct flb_input_instance *ins,
     if (o.type != MSGPACK_OBJECT_STR) {
         flb_plg_error(ins, "Invalid password_digest type message");
         flb_free(serverside);
+        flb_free(hostname);
         msgpack_unpacked_destroy(&result);
         return -1;
     }
@@ -1200,55 +1201,6 @@ error:
     return -1;
 }
 
-static size_t gzip_concatenated_count(const char *data, size_t len)
-{
-    int i;
-    size_t count = 0;
-    const uint8_t *p;
-
-    p = (const uint8_t *) data;
-
-    /* search other gzip starting bits and method. */
-    for (i = 2; i < len &&
-                 i + 2 <= len; i++) {
-        if (p[i] == 0x1F && p[i+1] == 0x8B && p[i+2] == 8) {
-            count++;
-        }
-    }
-
-    return count;
-}
-
-static size_t gzip_concatenated_borders(const char *data, size_t len, size_t **out_borders, size_t border_count)
-{
-    int i;
-    size_t count = 0;
-    const uint8_t *p;
-    size_t *borders = NULL;
-
-    p = (const uint8_t *) data;
-    borders = (size_t *) flb_calloc(1, sizeof(size_t) * (border_count + 1));
-    if (borders == NULL) {
-        flb_errno();
-        return -1;
-    }
-
-    /* search other gzip starting bits and method. */
-    for (i = 2; i < len &&
-                 i + 2 <= len; i++) {
-        if (p[i] == 0x1F && p[i+1] == 0x8B && p[i+2] == 8) {
-            borders[count] = i;
-            count++;
-        }
-    }
-    /* The length of the last border refers to the original length. */
-    borders[border_count] = len;
-
-    *out_borders = borders;
-
-    return count;
-}
-
 int fw_prot_process(struct flb_input_instance *ins, struct fw_conn *conn)
 {
     int ret;
@@ -1543,11 +1495,16 @@ int fw_prot_process(struct flb_input_instance *ins, struct fw_conn *conn)
                         size_t *gzip_borders = NULL;
                         const size_t original_len = len;
 
-                        gzip_payloads_count = gzip_concatenated_count(data, len);
+                        gzip_payloads_count = flb_gzip_count(data, len, NULL, 0);
                         flb_plg_debug(ctx->ins, "concatenated gzip payload count is %zd",
                                      gzip_payloads_count);
                         if (gzip_payloads_count > 0) {
-                            if (gzip_concatenated_borders(data, len, &gzip_borders, gzip_payloads_count) < 0) {
+                            gzip_borders = (size_t *)flb_calloc(1, sizeof(size_t) * (gzip_payloads_count + 1));
+                            if (gzip_borders == NULL) {
+                                flb_errno();
+                                return -1;
+                            }
+                            if (flb_gzip_count(data, len, &gzip_borders, gzip_payloads_count) < 0) {
                                 flb_plg_error(ctx->ins,
                                              "failed to traverse boundaries of concatenated gzip payloads");
                                 return -1;
@@ -1618,7 +1575,7 @@ int fw_prot_process(struct flb_input_instance *ins, struct fw_conn *conn)
                         /* a valid payload of gzip is larger than 18 bytes. */
                         if (gzip_payloads_count > 0) {
                             if ((gzip_payloads_count - loop) > 0 &&
-                                (original_len - gzip_borders[loop]) > 18) {
+                                (original_len - gzip_borders[loop]) >= 18) {
                                 len = original_len - gzip_borders[loop];
                                 flb_plg_debug(ctx->ins, "left unconsumed %zd byte(s)", len);
                                 prev_pos = gzip_borders[loop];
