@@ -24,6 +24,7 @@
 
 #include "http.h"
 #include "http_conn.h"
+#include "http_prot.h"
 #include "http_config.h"
 
 /*
@@ -92,25 +93,67 @@ static int in_http_init(struct flb_input_instance *ins,
 
     port = (unsigned short int) strtoul(ctx->tcp_port, NULL, 10);
 
-    ctx->downstream = flb_downstream_create(FLB_TRANSPORT_TCP,
-                                            ins->flags,
-                                            ctx->listen,
-                                            port,
-                                            ins->tls,
-                                            config,
-                                            &ins->net_setup);
+    if (ctx->enable_http2) {
+        ret = flb_http_server_init(&ctx->http_server, 
+                                    HTTP_PROTOCOL_AUTODETECT,
+                                    (FLB_HTTP_SERVER_FLAG_KEEPALIVE | FLB_HTTP_SERVER_FLAG_AUTO_INFLATE),
+                                    NULL,
+                                    ins->host.listen,
+                                    ins->host.port,
+                                    ins->tls,
+                                    ins->flags,
+                                    &ins->net_setup,
+                                    flb_input_event_loop_get(ins),
+                                    ins->config,
+                                    (void *) ctx);
 
-    if (ctx->downstream == NULL) {
-        flb_plg_error(ctx->ins,
-                      "could not initialize downstream on %s:%s. Aborting",
-                      ctx->listen, ctx->tcp_port);
+        if (ret != 0) {
+            flb_plg_error(ctx->ins,
+                          "could not initialize http server on %s:%u. Aborting",
+                          ins->host.listen, ins->host.port);
 
-        http_config_destroy(ctx);
+            http_config_destroy(ctx);
 
-        return -1;
+            return -1;
+        }
+
+        ret = flb_http_server_start(&ctx->http_server);
+
+        if (ret != 0) {
+            flb_plg_error(ctx->ins,
+                          "could not start http server on %s:%u. Aborting",
+                          ins->host.listen, ins->host.port);
+
+            http_config_destroy(ctx);
+
+            return -1;
+        }
+
+        ctx->http_server.request_callback = http_prot_handle_ng;
+
+        flb_input_downstream_set(ctx->http_server.downstream, ctx->ins);
     }
+    else {
+        ctx->downstream = flb_downstream_create(FLB_TRANSPORT_TCP,
+                                                ins->flags,
+                                                ctx->listen,
+                                                port,
+                                                ins->tls,
+                                                config,
+                                                &ins->net_setup);
 
-    flb_input_downstream_set(ctx->downstream, ctx->ins);
+        if (ctx->downstream == NULL) {
+            flb_plg_error(ctx->ins,
+                        "could not initialize downstream on %s:%s. Aborting",
+                        ctx->listen, ctx->tcp_port);
+
+            http_config_destroy(ctx);
+
+            return -1;
+        }
+
+        flb_input_downstream_set(ctx->downstream, ctx->ins);
+    }
 
     if (ctx->successful_response_code != 200 &&
         ctx->successful_response_code != 201 &&
@@ -120,20 +163,22 @@ static int in_http_init(struct flb_input_instance *ins,
         ctx->successful_response_code = 201;
     }
 
-    /* Collect upon data available on the standard input */
-    ret = flb_input_set_collector_socket(ins,
-                                         in_http_collect,
-                                         ctx->downstream->server_fd,
-                                         config);
-    if (ret == -1) {
-        flb_plg_error(ctx->ins, "Could not set collector for IN_TCP input plugin");
-        http_config_destroy(ctx);
+    if (!ctx->enable_http2) {
+        /* Collect upon data available on the standard input */
+        ret = flb_input_set_collector_socket(ins,
+                                            in_http_collect,
+                                            ctx->downstream->server_fd,
+                                            config);
+        if (ret == -1) {
+            flb_plg_error(ctx->ins, "Could not set collector for IN_TCP input plugin");
+            http_config_destroy(ctx);
 
-        return -1;
+            return -1;
+        }
+
+        ctx->collector_id = ret;
     }
 
-    ctx->collector_id = ret;
-
     return 0;
 }
 
@@ -154,6 +199,12 @@ static int in_http_exit(void *data, struct flb_config *config)
 
 /* Configuration properties map */
 static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_BOOL, "http2", "true",
+     0, FLB_TRUE, offsetof(struct flb_http, enable_http2),
+     NULL
+    },
+
     {
      FLB_CONFIG_MAP_SIZE, "buffer_max_size", HTTP_BUFFER_MAX_SIZE,
      0, FLB_TRUE, offsetof(struct flb_http, buffer_max_size),
