@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,6 +20,7 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_input.h>
 #include <fluent-bit/flb_input_plugin.h>
+#include <fluent-bit/flb_log_event.h>
 #include <fluent-bit/flb_kernel.h>
 #include <fluent-bit/flb_pack.h>
 
@@ -120,10 +120,8 @@ static int in_mem_init(struct flb_input_instance *in,
                        struct flb_config *config, void *data)
 {
     int ret;
-    const char *tmp;
     struct flb_in_mem_config *ctx;
     (void) data;
-    const char *pval = NULL;
 
     /* Initialize context */
     ctx = flb_malloc(sizeof(struct flb_in_mem_config));
@@ -134,21 +132,20 @@ static int in_mem_init(struct flb_input_instance *in,
     ctx->pid = 0;
     ctx->page_size = sysconf(_SC_PAGESIZE);
     ctx->ins = in;
+    
+    /* Load the config map */
+    ret = flb_input_config_map_set(in, (void *)ctx);
+    if (ret == -1) {
+        flb_free(ctx);
+        return -1;
+    }
 
     /* Collection time setting */
-    pval = flb_input_get_property("interval_sec", in);
-    if (pval != NULL && atoi(pval) > 0) {
-        ctx->interval_sec = atoi(pval);
+    if (ctx->interval_sec <= 0) {
+        ctx->interval_sec = atoi(DEFAULT_INTERVAL_SEC);
     }
-    else {
-        ctx->interval_sec = DEFAULT_INTERVAL_SEC;
-    }
-    ctx->interval_nsec = DEFAULT_INTERVAL_NSEC;
-
-    /* Check if the caller want's to trace a specific Process ID */
-    tmp = flb_input_get_property("pid", in);
-    if (tmp) {
-        ctx->pid = atoi(tmp);
+    if (ctx->interval_nsec <= 0) {
+        ctx->interval_nsec = atoi(DEFAULT_INTERVAL_NSEC);
     }
 
     /* Set the context */
@@ -162,6 +159,16 @@ static int in_mem_init(struct flb_input_instance *in,
                                        config);
     if (ret == -1) {
         flb_plg_error(ctx->ins, "could not set collector for memory input plugin");
+        return -1;
+    }
+
+    ret = flb_log_event_encoder_init(&ctx->log_encoder,
+                                     FLB_LOG_EVENT_FORMAT_DEFAULT);
+
+    if (ret != FLB_EVENT_ENCODER_SUCCESS) {
+        flb_plg_error(ctx->ins, "error initializing event encoder : %d", ret);
+
+        return -1;
     }
 
     return 0;
@@ -171,13 +178,9 @@ static int in_mem_collect(struct flb_input_instance *i_ins,
                           struct flb_config *config, void *in_context)
 {
     int ret;
-    int len;
-    int entries = 6;/* (total,used,free) * (memory, swap) */
     struct proc_task *task = NULL;
     struct flb_in_mem_config *ctx = in_context;
     struct flb_in_mem_info info;
-    msgpack_packer mp_pck;
-    msgpack_sbuffer mp_sbuf;
 
     if (ctx->pid) {
         task = proc_stat(ctx->pid, ctx->page_size);
@@ -196,56 +199,46 @@ static int in_mem_collect(struct flb_input_instance *i_ins,
         return -1;
     }
 
-    if (task) {
-        entries += 2;
+    ret = flb_log_event_encoder_begin_record(&ctx->log_encoder);
+
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        ret = flb_log_event_encoder_set_current_timestamp(
+                &ctx->log_encoder);
     }
 
-    /* Initialize local msgpack buffer */
-    msgpack_sbuffer_init(&mp_sbuf);
-    msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        ret = flb_log_event_encoder_append_body_values(
+                &ctx->log_encoder,
+                FLB_LOG_EVENT_CSTRING_VALUE("Mem.total"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.mem_total),
 
-    /* Pack the data */
-    msgpack_pack_array(&mp_pck, 2);
-    flb_pack_time_now(&mp_pck);
-    msgpack_pack_map(&mp_pck, entries);
+                FLB_LOG_EVENT_CSTRING_VALUE("Mem.used"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.mem_used),
 
-    msgpack_pack_str(&mp_pck, 9);
-    msgpack_pack_str_body(&mp_pck, "Mem.total", 9);
-    msgpack_pack_uint64(&mp_pck, info.mem_total);
+                FLB_LOG_EVENT_CSTRING_VALUE("Mem.free"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.mem_free),
 
-    msgpack_pack_str(&mp_pck, 8);
-    msgpack_pack_str_body(&mp_pck, "Mem.used", 8);
-    msgpack_pack_uint64(&mp_pck, info.mem_used);
+                FLB_LOG_EVENT_CSTRING_VALUE("Swap.total"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.swap_total),
 
-    msgpack_pack_str(&mp_pck, 8);
-    msgpack_pack_str_body(&mp_pck, "Mem.free", 8);
-    msgpack_pack_uint64(&mp_pck, info.mem_free);
+                FLB_LOG_EVENT_CSTRING_VALUE("Swap.used"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.swap_used),
 
-    msgpack_pack_str(&mp_pck, 10);
-    msgpack_pack_str_body(&mp_pck, "Swap.total", 10);
-    msgpack_pack_uint64(&mp_pck, info.swap_total);
+                FLB_LOG_EVENT_CSTRING_VALUE("Swap.free"),
+                FLB_LOG_EVENT_UINT64_VALUE(info.swap_free));
+    }
 
-    msgpack_pack_str(&mp_pck, 9);
-    msgpack_pack_str_body(&mp_pck, "Swap.used", 9);
-    msgpack_pack_uint64(&mp_pck, info.swap_used);
-
-    msgpack_pack_str(&mp_pck, 9);
-    msgpack_pack_str_body(&mp_pck, "Swap.free", 9);
-    msgpack_pack_uint64(&mp_pck, info.swap_free);
-
-
-    if (task) {
+    if (task != NULL &&
+        ret == FLB_EVENT_ENCODER_SUCCESS) {
         /* RSS bytes */
-        msgpack_pack_str(&mp_pck, 10);
-        msgpack_pack_str_body(&mp_pck, "proc_bytes", 10);
-        msgpack_pack_uint64(&mp_pck, task->proc_rss);
 
-        /* RSS Human readable format */
-        len = strlen(task->proc_rss_hr);
-        msgpack_pack_str(&mp_pck, 7);
-        msgpack_pack_str_body(&mp_pck, "proc_hr", 7);
-        msgpack_pack_str(&mp_pck, len);
-        msgpack_pack_str_body(&mp_pck, task->proc_rss_hr, len);
+        ret = flb_log_event_encoder_append_body_values(
+                &ctx->log_encoder,
+                FLB_LOG_EVENT_CSTRING_VALUE("proc_bytes"),
+                FLB_LOG_EVENT_UINT64_VALUE(task->proc_rss),
+
+                FLB_LOG_EVENT_CSTRING_VALUE("proc_hr"),
+                FLB_LOG_EVENT_UINT64_VALUE(task->proc_rss_hr));
 
         proc_free(task);
     }
@@ -256,8 +249,24 @@ static int in_mem_collect(struct flb_input_instance *i_ins,
                   info.swap_total, info.swap_used, info.swap_free);
     ++ctx->idx;
 
-    flb_input_chunk_append_raw(i_ins, NULL, 0, mp_sbuf.data, mp_sbuf.size);
-    msgpack_sbuffer_destroy(&mp_sbuf);
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        ret = flb_log_event_encoder_commit_record(&ctx->log_encoder);
+    }
+
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        flb_input_log_append(i_ins, NULL, 0,
+                             ctx->log_encoder.output_buffer,
+                             ctx->log_encoder.output_length);
+
+        ret = 0;
+    }
+    else {
+        flb_plg_error(i_ins, "Error encoding record : %d", ret);
+
+        ret = -1;
+    }
+
+    flb_log_event_encoder_reset(&ctx->log_encoder);
 
     return 0;
 }
@@ -271,12 +280,34 @@ static int in_mem_exit(void *data, struct flb_config *config)
         return 0;
     }
 
+    flb_log_event_encoder_destroy(&ctx->log_encoder);
+
     /* done */
     flb_free(ctx);
 
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    {
+      FLB_CONFIG_MAP_INT, "interval_sec", DEFAULT_INTERVAL_SEC,
+      0, FLB_TRUE, offsetof(struct flb_in_mem_config, interval_sec),
+      "Set the collector interval"
+    },
+    {
+      FLB_CONFIG_MAP_INT, "interval_nsec", DEFAULT_INTERVAL_NSEC,
+      0, FLB_TRUE, offsetof(struct flb_in_mem_config, interval_nsec),
+      "Set the collector interval (subseconds)"
+    },
+    {
+      FLB_CONFIG_MAP_INT, "pid", "0",
+      0, FLB_TRUE, offsetof(struct flb_in_mem_config, pid),
+      "Set the PID of the process to measure"
+    },
+    /* EOF */
+    {0}
+};
+
 struct flb_input_plugin in_mem_plugin = {
     .name         = "mem",
     .description  = "Memory Usage",
@@ -284,5 +315,6 @@ struct flb_input_plugin in_mem_plugin = {
     .cb_pre_run   = NULL,
     .cb_collect   = in_mem_collect,
     .cb_flush_buf = NULL,
-    .cb_exit      = in_mem_exit
+    .cb_exit      = in_mem_exit,
+    .config_map   = config_map
 };
