@@ -107,6 +107,9 @@ static int collect_metrics(struct prom_scrape *ctx)
         flb_http_bearer_auth(c, ctx->bearer_token);
     }
 
+    /* Add User-Agent */
+    flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
+
     ret = flb_http_do(c, &b_sent);
     if (ret != 0) {
         flb_plg_error(ctx->ins, "http do error");
