@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,6 +22,8 @@
 
 #include <fluent-bit/flb_config.h>
 #include <fluent-bit/flb_input.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
 #define DEFAULT_BUF_SIZE 16000
 
@@ -33,11 +34,13 @@ struct flb_in_stdin_config {
     size_t buf_size;          /* size of a buffer      */
     int    buf_len;           /* read buffer length    */
     char   *buf;              /* read buffer           */
+    flb_sds_t parser_name;    /* name of the parser    */
 
     /* Parser / Format */
     struct flb_parser *parser;
     struct flb_pack_state pack_state;
     struct flb_input_instance *ins;
+    struct flb_log_event_encoder *log_encoder;
 };
 
 extern struct flb_input_plugin in_stdin_plugin;
