@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,11 +20,12 @@
 #include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_unescape.h>
 #include <fluent-bit/flb_jsmn.h>
+#include <fluent-bit/flb_utils.h>
+#include <fluent-bit/flb_aws_credentials.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <fluent-bit/flb_utils.h>
 
 #include "bigquery.h"
 #include "bigquery_conf.h"
@@ -176,6 +176,7 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
 {
     int ret;
     const char *tmp;
+    char *tmp_aws_region;
     struct flb_bigquery *ctx;
     struct flb_bigquery_oauth_credentials *creds;
 
@@ -188,6 +189,13 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
     ctx->ins = ins;
     ctx->config = config;
 
+    ret = flb_output_config_map_set(ins, (void *)ctx);
+    if (ret == -1) {
+        flb_plg_error(ins, "unable to load configuration");
+        flb_free(ctx);
+        return NULL;
+    }
+
     /* Lookup credentials file */
     creds = flb_calloc(1, sizeof(struct flb_bigquery_oauth_credentials));
     if (!creds) {
@@ -197,17 +205,55 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
     }
     ctx->oauth_credentials = creds;
 
-    tmp = flb_output_get_property("google_service_credentials", ins);
-    if (tmp) {
-        ctx->credentials_file = flb_sds_create(tmp);
-    }
-    else {
+    if (ctx->credentials_file == NULL) {
         tmp = getenv("GOOGLE_SERVICE_CREDENTIALS");
         if (tmp) {
             ctx->credentials_file = flb_sds_create(tmp);
         }
     }
 
+    if (ctx->credentials_file && ctx->has_identity_federation) {
+        flb_plg_error(ctx->ins, "Either `google_service_credentials` or `enable_identity_federation` should be set");
+        return NULL;
+    }
+
+    if (ctx->aws_region) {
+        tmp_aws_region = flb_aws_endpoint("sts", ctx->aws_region);
+        if (!tmp_aws_region) {
+            flb_plg_error(ctx->ins, "Could not create AWS STS regional endpoint");
+            return NULL;
+        }
+        ctx->aws_sts_endpoint = flb_sds_create(tmp_aws_region);
+        flb_free(tmp_aws_region);
+    }
+
+    if (ctx->has_identity_federation) {
+        if (!ctx->aws_region) {
+            flb_plg_error(ctx->ins, "`aws_region` is required when `enable_identity_federation` is true");
+            return NULL;
+        }
+
+        if (!ctx->project_number) {
+            flb_plg_error(ctx->ins, "`project_number` is required when `enable_identity_federation` is true");
+            return NULL;
+        }
+
+        if (!ctx->pool_id) {
+            flb_plg_error(ctx->ins, "`pool_id` is required when `enable_identity_federation` is true");
+            return NULL;
+        }
+
+        if (!ctx->provider_id) {
+            flb_plg_error(ctx->ins, "`provider_id` is required when `enable_identity_federation` is true");
+            return NULL;
+        }
+
+        if (!ctx->google_service_account) {
+            flb_plg_error(ctx->ins, "`google_service_account` is required when `enable_identity_federation` is true");
+            return NULL;
+        }
+    }
+
     if (ctx->credentials_file) {
         ret = flb_bigquery_read_credentials_file(ctx,
                                                  ctx->credentials_file,
@@ -217,7 +263,7 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
             return NULL;
         }
     }
-    else {
+    else if (!ctx->credentials_file && !ctx->has_identity_federation) {
         /*
          * If no credentials file has been defined, do manual lookup of the
          * client email and the private key.
@@ -248,26 +294,26 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
         }
 
         if (!creds->client_email) {
-            flb_plg_error(ctx->ins, "client_email is not defined");
+            flb_plg_error(ctx->ins, "service_account_email/client_email is not defined");
             flb_bigquery_conf_destroy(ctx);
             return NULL;
         }
 
         if (!creds->private_key) {
-            flb_plg_error(ctx->ins, "private_key is not defined");
+            flb_plg_error(ctx->ins, "service_account_secret/private_key is not defined");
             flb_bigquery_conf_destroy(ctx);
             return NULL;
         }
     }
 
     /* config: 'project_id' */
-    tmp = flb_output_get_property("project_id", ins);
-    if (tmp) {
-        ctx->project_id = flb_sds_create(tmp);
-    }
-    else {
-       if (creds->project_id) {
-            ctx->project_id = flb_sds_create(creds->project_id);
+    if (ctx->project_id == NULL) {
+        if (creds->project_id) {
+            /* flb_config_map_destroy uses the pointer within the config_map struct to
+             * free the value so if we assign it here it is safe to free later with the
+             * creds struct. If we do not we will leak here.
+             */
+            ctx->project_id = creds->project_id;
             if (!ctx->project_id) {
                 flb_plg_error(ctx->ins,
                               "failed extracting 'project_id' from credentials.");
@@ -284,45 +330,19 @@ struct flb_bigquery *flb_bigquery_conf_create(struct flb_output_instance *ins,
     }
 
     /* config: 'dataset_id' */
-    tmp = flb_output_get_property("dataset_id", ins);
-    if (tmp) {
-        ctx->dataset_id = flb_sds_create(tmp);
-    }
-    else {
+    if (ctx->dataset_id == NULL) {
         flb_plg_error(ctx->ins, "property 'dataset_id' is not defined");
         flb_bigquery_conf_destroy(ctx);
         return NULL;
     }
 
     /* config: 'table_id' */
-    tmp = flb_output_get_property("table_id", ins);
-    if (tmp) {
-        ctx->table_id = flb_sds_create(tmp);
-    }
-    else {
+    if (ctx->table_id == NULL) {
         flb_plg_error(ctx->ins, "property 'table_id' is not defined");
         flb_bigquery_conf_destroy(ctx);
         return NULL;
     }
 
-    /* config: 'skip_invalid_rows' */
-    tmp = flb_output_get_property("skip_invalid_rows", ins);
-    if (tmp && flb_utils_bool(tmp)) {
-        ctx->skip_invalid_rows = FLB_TRUE;
-    }
-    else {
-        ctx->skip_invalid_rows = FLB_FALSE;
-    }
-
-    /* config: 'ignore_unknown_values' */
-    tmp = flb_output_get_property("ignore_unknown_values", ins);
-    if (tmp && flb_utils_bool(tmp)) {
-        ctx->ignore_unknown_values = FLB_TRUE;
-    }
-    else {
-        ctx->ignore_unknown_values = FLB_FALSE;
-    }
-
     /* Create the target URI */
     ctx->uri = flb_sds_create_size(sizeof(FLB_BIGQUERY_RESOURCE_TEMPLATE)-6 +
                                    flb_sds_len(ctx->project_id) +
@@ -368,13 +388,42 @@ int flb_bigquery_conf_destroy(struct flb_bigquery *ctx)
         return -1;
     }
 
-    flb_sds_destroy(ctx->credentials_file);
-
     flb_bigquery_oauth_credentials_destroy(ctx->oauth_credentials);
 
-    flb_sds_destroy(ctx->project_id);
-    flb_sds_destroy(ctx->dataset_id);
-    flb_sds_destroy(ctx->table_id);
+    if (ctx->aws_sts_upstream) {
+        flb_upstream_destroy(ctx->aws_sts_upstream);
+    }
+
+    if (ctx->google_sts_upstream) {
+        flb_upstream_destroy(ctx->google_sts_upstream);
+    }
+
+    if (ctx->google_iam_upstream) {
+        flb_upstream_destroy(ctx->google_iam_upstream);
+    }
+
+    if (ctx->aws_provider) {
+        flb_aws_provider_destroy(ctx->aws_provider);
+    }
+
+    if (ctx->aws_tls) {
+        flb_tls_destroy(ctx->aws_tls);
+    }
+
+    if (ctx->aws_sts_tls) {
+        flb_tls_destroy(ctx->aws_sts_tls);
+    }
+
+    if (ctx->google_sts_tls) {
+        flb_tls_destroy(ctx->google_sts_tls);
+    }
+
+    if (ctx->google_iam_tls) {
+        flb_tls_destroy(ctx->google_iam_tls);
+    }
+
+    flb_sds_destroy(ctx->aws_sts_endpoint);
+    flb_sds_destroy(ctx->sa_token);
     flb_sds_destroy(ctx->uri);
 
     if (ctx->o) {
