@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,6 +22,7 @@
 #include <fluent-bit/flb_http_client.h>
 #include <fluent-bit/flb_time.h>
 #include <fluent-bit/flb_metrics.h>
+#include <fluent-bit/flb_log_event_decoder.h>
 
 #include <msgpack.h>
 
@@ -66,41 +66,24 @@ static char *influxdb_format(const char *tag, int tag_len,
     int ret;
     int n_size;
     uint64_t seq = 0;
-    size_t off = 0;
     char *buf;
     char *str = NULL;
     size_t str_size;
     char tmp[128];
-    msgpack_unpacked result;
-    msgpack_object root;
     msgpack_object map;
-    msgpack_object *obj;
     struct flb_time tm;
     struct influxdb_bulk *bulk = NULL;
     struct influxdb_bulk *bulk_head = NULL;
     struct influxdb_bulk *bulk_body = NULL;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
 
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
 
-    /* Iterate the original buffer and perform adjustments */
-    msgpack_unpacked_init(&result);
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
 
-    /* Perform some format validation */
-    ret = msgpack_unpack_next(&result, data, bytes, &off);
-    if (!ret) {
-        return NULL;
-    }
-
-    /* We 'should' get an array */
-    if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-        /*
-         * If we got a different format, we assume the caller knows what he is
-         * doing, we just duplicate the content in a new buffer and cleanup.
-         */
-        return NULL;
-    }
-
-    root = result.data;
-    if (root.via.array.size == 0) {
         return NULL;
     }
 
@@ -120,23 +103,12 @@ static char *influxdb_format(const char *tag, int tag_len,
         goto error;
     }
 
-    off = 0;
-    msgpack_unpacked_destroy(&result);
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
+        flb_time_copy(&tm, &log_event.timestamp);
 
-        /* Each array must have two entries: time and record */
-        root = result.data;
-        if (root.via.array.size != 2) {
-            continue;
-        }
-
-
-        flb_time_pop_from_msgpack(&tm, &result, &obj);
-        map    = root.via.array.ptr[1];
+        map    = *log_event.body;
         n_size = map.via.map.size + 1;
 
         seq = ctx->seq;
@@ -294,7 +266,7 @@ static char *influxdb_format(const char *tag, int tag_len,
         bulk_body->len = 0;
     }
 
-    msgpack_unpacked_destroy(&result);
+    flb_log_event_decoder_destroy(&log_decoder);
 
     *out_size = bulk->len;
     buf = bulk->ptr;
@@ -320,7 +292,9 @@ error:
     if (bulk_body != NULL) {
         influxdb_bulk_destroy(bulk_body);
     }
-    msgpack_unpacked_destroy(&result);
+
+    flb_log_event_decoder_destroy(&log_decoder);
+
     return NULL;
 }
 
@@ -442,7 +416,7 @@ static int format_metrics(struct flb_output_instance *ins,
 {
     int ret;
     size_t off = 0;
-    cmt_sds_t text;
+    cfl_sds_t text;
     struct cmt *cmt = NULL;
 
     /* get cmetrics context */
@@ -468,20 +442,21 @@ static int format_metrics(struct flb_output_instance *ins,
     return 0;
 }
 
-static void cb_influxdb_flush(const void *data, size_t bytes,
-                              const char *tag, int tag_len,
+static void cb_influxdb_flush(struct flb_event_chunk *event_chunk,
+                              struct flb_output_flush *out_flush,
                               struct flb_input_instance *i_ins,
                               void *out_context,
                               struct flb_config *config)
 {
     int ret;
+    int out_ret = FLB_OK;
     int is_metric = FLB_FALSE;
     size_t b_sent;
     size_t bytes_out;
     char *pack;
     char tmp[128];
     struct mk_list *head;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
     struct flb_http_client *c;
     struct flb_config_map_val *mv;
     struct flb_slist_entry *key = NULL;
@@ -489,9 +464,11 @@ static void cb_influxdb_flush(const void *data, size_t bytes,
     struct flb_influxdb *ctx = out_context;
 
     /* Convert format: metrics / logs */
-    if (flb_input_event_type_is_metric(i_ins)) {
+    if (event_chunk->type == FLB_EVENT_TYPE_METRICS) {
         /* format metrics */
-        ret = format_metrics(ctx->ins, (char *) data, bytes,
+        ret = format_metrics(ctx->ins,
+                             (char *) event_chunk->data,
+                             event_chunk->size,
                              &pack, &bytes_out);
         if (ret == -1) {
             FLB_OUTPUT_RETURN(FLB_ERROR);
@@ -500,7 +477,9 @@ static void cb_influxdb_flush(const void *data, size_t bytes,
     }
     else {
         /* format logs */
-        pack = influxdb_format(tag, tag_len, data, bytes, &bytes_out, ctx);
+        pack = influxdb_format(event_chunk->tag, flb_sds_len(event_chunk->tag),
+                               event_chunk->data, event_chunk->size,
+                               &bytes_out, ctx);
         if (!pack) {
             FLB_OUTPUT_RETURN(FLB_ERROR);
         }
@@ -559,7 +538,8 @@ static void cb_influxdb_flush(const void *data, size_t bytes,
         flb_plg_debug(ctx->ins, "http_do=%i OK", ret);
     }
     else {
-        flb_plg_warn(ctx->ins, "http_do=%i", ret);
+        flb_plg_error(ctx->ins, "http_do=%i", ret);
+        out_ret = FLB_RETRY;
     }
 
     flb_http_client_destroy(c);
@@ -574,7 +554,7 @@ static void cb_influxdb_flush(const void *data, size_t bytes,
     /* Release the connection */
     flb_upstream_conn_release(u_conn);
 
-    FLB_OUTPUT_RETURN(FLB_OK);
+    FLB_OUTPUT_RETURN(out_ret);
 }
 
 static int cb_influxdb_exit(void *data, struct flb_config *config)
