@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -42,7 +41,7 @@ struct flush {
     size_t data_size;
 
     /* log records- each of these has a pointer to their message in tmp_buf */
-    struct event *events;
+    struct firehose_event *events;
     int events_capacity;
     /* current event */
     int event_index;
@@ -59,7 +58,7 @@ struct flush {
     int records_processed;
 };
 
-struct event {
+struct firehose_event {
     char *json;
     size_t len;
     struct timespec timestamp;
@@ -88,8 +87,10 @@ struct flb_firehose {
     const char *log_key;
     const char *external_id;
     char *sts_endpoint;
+    char *profile;
     int custom_endpoint;
     int retry_requests;
+    int compression;
 
     /* must be freed on shutdown if custom_endpoint is not set */
     char *endpoint;
