@@ -59,6 +59,9 @@ struct opentelemetry_context {
     char *host;
     int port;
 
+    /* record metadata parsing */
+    flb_sds_t logs_metadata_key;
+
     /* metadata keys */
     flb_sds_t logs_observed_timestamp_metadata_key;
     struct flb_record_accessor *ra_observed_timestamp_metadata;
@@ -139,6 +142,31 @@ struct opentelemetry_context {
 
     /* Compression mode (gzip) */
     int compress_gzip;
+
+    /* FLB/OTLP Record accessor patterns */
+    struct flb_record_accessor *ra_meta_schema;
+    struct flb_record_accessor *ra_meta_resource_id;
+    struct flb_record_accessor *ra_meta_scope_id;
+    struct flb_record_accessor *ra_resource_attr;
+    struct flb_record_accessor *ra_resource_schema_url;
+
+    struct flb_record_accessor *ra_scope_name;
+    struct flb_record_accessor *ra_scope_version;
+    struct flb_record_accessor *ra_scope_attr;
+
+    /* log: metadata components coming from OTLP */
+    struct flb_record_accessor *ra_log_meta_otlp_observed_ts;
+    struct flb_record_accessor *ra_log_meta_otlp_timestamp;
+    struct flb_record_accessor *ra_log_meta_otlp_severity_number;
+    struct flb_record_accessor *ra_log_meta_otlp_severity_text;
+    struct flb_record_accessor *ra_log_meta_otlp_attr;
+    struct flb_record_accessor *ra_log_meta_otlp_trace_id;
+    struct flb_record_accessor *ra_log_meta_otlp_span_id;
+    struct flb_record_accessor *ra_log_meta_otlp_trace_flags;
 };
 
+int opentelemetry_http_post(struct opentelemetry_context *ctx,
+                            const void *body, size_t body_len,
+                            const char *tag, int tag_len,
+                            const char *uri);
 #endif
