@@ -22,6 +22,7 @@
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_kv.h>
+#include <fluent-bit/flb_record_accessor.h>
 
 #include "opentelemetry.h"
 #include "opentelemetry_conf.h"
@@ -415,7 +416,7 @@ struct opentelemetry_context *flb_opentelemetry_context_create(struct flb_output
     if (ctx->ra_trace_id_metadata == NULL) {
         flb_plg_error(ins, "failed to create ra for trace id");
     }
-    ctx->ra_attributes_metadata = flb_ra_create((char*)ctx->logs_attributes_metadata_key,
+    ctx->ra_attributes_metadata = flb_ra_create((char*) ctx->logs_attributes_metadata_key,
                                                 FLB_FALSE);
     if (ctx->ra_attributes_metadata == NULL) {
         flb_plg_error(ins, "failed to create ra for attributes");
@@ -441,6 +442,90 @@ struct opentelemetry_context *flb_opentelemetry_context_create(struct flb_output
         flb_plg_error(ins, "failed to create ra for message severity number");
     }
 
+    /* record accessor: group metadata */
+    ctx->ra_meta_schema = flb_ra_create("$schema", FLB_FALSE);
+    if (ctx->ra_meta_schema == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for schema");
+    }
+
+    ctx->ra_meta_resource_id = flb_ra_create((char *) "$resource_id", FLB_FALSE);
+    if (ctx->ra_meta_resource_id == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for resource_id");
+    }
+
+    ctx->ra_meta_scope_id = flb_ra_create((char *) "$scope_id", FLB_FALSE);
+    if (ctx->ra_meta_scope_id == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for scope_id");
+    }
+
+    /* record accessor: group body */
+    ctx->ra_resource_attr = flb_ra_create("$resource['attributes']", FLB_FALSE);
+    if (ctx->ra_resource_attr == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for resource attributes");
+    }
+
+    ctx->ra_resource_schema_url = flb_ra_create("$schema_url", FLB_FALSE);
+    if (ctx->ra_resource_schema_url == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for resource schema url");
+    }
+
+    ctx->ra_scope_name = flb_ra_create("$scope['name']", FLB_FALSE);
+    if (ctx->ra_scope_name == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for scope name");
+    }
+
+    ctx->ra_scope_version = flb_ra_create("$scope['version']", FLB_FALSE);
+    if (ctx->ra_scope_version == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for scope version");
+    }
+
+    ctx->ra_scope_attr = flb_ra_create("$scope['attributes']", FLB_FALSE);
+    if (ctx->ra_scope_attr == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for scope attributes");
+    }
+
+    /* log metadata under $otlp (set by in_opentelemetry) */
+
+    ctx->ra_log_meta_otlp_observed_ts = flb_ra_create("$otlp['observed_timestamp']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_observed_ts == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp observed timestamp");
+    }
+
+    ctx->ra_log_meta_otlp_timestamp = flb_ra_create("$otlp['timestamp']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_timestamp == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp timestamp");
+    }
+
+    ctx->ra_log_meta_otlp_severity_number = flb_ra_create("$otlp['severity_number']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_severity_number == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp severity number");
+    }
+
+    ctx->ra_log_meta_otlp_severity_text = flb_ra_create("$otlp['severity_text']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_severity_text == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp severity text");
+    }
+
+    ctx->ra_log_meta_otlp_attr = flb_ra_create("$otlp['attributes']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_attr == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp attributes");
+    }
+
+    ctx->ra_log_meta_otlp_trace_id = flb_ra_create("$otlp['trace_id']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_trace_id == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp trace id");
+    }
+
+    ctx->ra_log_meta_otlp_span_id = flb_ra_create("$otlp['span_id']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_span_id == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp span id");
+    }
+
+    ctx->ra_log_meta_otlp_trace_flags = flb_ra_create("$otlp['trace_flags']", FLB_FALSE);
+    if (ctx->ra_log_meta_otlp_trace_flags == NULL) {
+        flb_plg_error(ins, "failed to create record accessor for otlp trace flags");
+    }
+
     return ctx;
 }
 
@@ -499,6 +584,63 @@ void flb_opentelemetry_context_destroy(struct opentelemetry_context *ctx)
         flb_ra_destroy(ctx->ra_severity_number_message);
     }
 
+    if (ctx->ra_meta_schema) {
+        flb_ra_destroy(ctx->ra_meta_schema);
+    }
+    if (ctx->ra_meta_resource_id) {
+        flb_ra_destroy(ctx->ra_meta_resource_id);
+    }
+    if (ctx->ra_meta_scope_id) {
+        flb_ra_destroy(ctx->ra_meta_scope_id);
+    }
+    if (ctx->ra_resource_attr) {
+        flb_ra_destroy(ctx->ra_resource_attr);
+    }
+    if (ctx->ra_resource_schema_url) {
+        flb_ra_destroy(ctx->ra_resource_schema_url);
+    }
+    if (ctx->ra_scope_name) {
+        flb_ra_destroy(ctx->ra_scope_name);
+    }
+    if (ctx->ra_scope_version) {
+        flb_ra_destroy(ctx->ra_scope_version);
+    }
+    if (ctx->ra_scope_attr) {
+        flb_ra_destroy(ctx->ra_scope_attr);
+    }
+
+    if (ctx->ra_log_meta_otlp_observed_ts) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_observed_ts);
+    }
+
+    if (ctx->ra_log_meta_otlp_timestamp) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_timestamp);
+    }
+
+    if (ctx->ra_log_meta_otlp_severity_number) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_severity_number);
+    }
+
+    if (ctx->ra_log_meta_otlp_severity_text) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_severity_text);
+    }
+
+    if (ctx->ra_log_meta_otlp_attr) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_attr);
+    }
+
+    if (ctx->ra_log_meta_otlp_trace_id) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_trace_id);
+    }
+
+    if (ctx->ra_log_meta_otlp_span_id) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_span_id);
+    }
+
+    if (ctx->ra_log_meta_otlp_trace_flags) {
+        flb_ra_destroy(ctx->ra_log_meta_otlp_trace_flags);
+    }
+
     flb_free(ctx->proxy_host);
     flb_free(ctx);
 }
