@@ -404,7 +404,7 @@ static int run_action_insert(struct content_modifier_ctx *ctx,
     ret = cfl_kvlist_insert_string_s(kvlist, key, cfl_sds_len(key), value, cfl_sds_len(value),
                                      CFL_FALSE);
     if (ret != 0) {
-        printf("Failed to insert key: %s\n", key);
+        flb_plg_debug(ctx->ins, "[action: insert] failed to insert key: %s", key);
         return -1;
     }
     return 0;
@@ -451,7 +451,10 @@ static int run_action_delete(struct content_modifier_ctx *ctx,
         return 0;
     }
 
-    return -1;
+    flb_plg_debug(ctx->ins, "[action: delete] key '%s' not found", key);
+
+    /* if the kvpair was not found, it's ok, we return zero */
+    return 0;
 }
 
 static int run_action_rename(struct content_modifier_ctx *ctx,
@@ -465,7 +468,8 @@ static int run_action_rename(struct content_modifier_ctx *ctx,
     /* if the kv pair already exists, remove it from the list */
     kvpair = cfl_object_kvpair_get(obj, key);
     if (!kvpair) {
-        return -1;
+        flb_plg_debug(ctx->ins, "[action: rename] key '%s' not found", key);
+        return 0;
     }
 
     tmp = kvpair->key;
@@ -595,6 +599,134 @@ static int run_action_convert(struct content_modifier_ctx *ctx,
     return 0;
 }
 
+static struct cfl_variant *otel_get_or_create_attributes(struct cfl_kvlist *kvlist)
+{
+    int ret;
+    struct cfl_list *head;
+    struct cfl_list *tmp;
+    struct cfl_kvpair *kvpair;
+    struct cfl_variant *val;
+    struct cfl_kvlist *kvlist_tmp;
+
+    /* iterate resource to find the attributes field */
+    cfl_list_foreach_safe(head, tmp, &kvlist->list) {
+        kvpair = cfl_list_entry(head, struct cfl_kvpair, _head);
+        if (cfl_sds_len(kvpair->key) != 10) {
+            continue;
+        }
+
+        if (strncmp(kvpair->key, "attributes", 10) == 0) {
+            val = kvpair->val;
+            if (val->type != CFL_VARIANT_KVLIST) {
+                return NULL;
+            }
+
+            return val;
+        }
+    }
+
+    /* create an empty kvlist as the value of attributes */
+    kvlist_tmp = cfl_kvlist_create();
+    if (!kvlist_tmp) {
+        return NULL;
+    }
+
+    /* create the attributes kvpair */
+    ret = cfl_kvlist_insert_kvlist_s(kvlist, "attributes", 10, kvlist_tmp);
+    if (ret != 0) {
+        cfl_kvlist_destroy(kvlist_tmp);
+        return NULL;
+    }
+
+    /* get the last kvpair from the list */
+    kvpair = cfl_list_entry_last(&kvlist->list, struct cfl_kvpair, _head);
+    if (!kvpair) {
+        return NULL;
+    }
+
+    return kvpair->val;
+}
+
+
+static struct cfl_variant *otel_get_attributes(int context, struct flb_mp_chunk_record *record)
+{
+    int key_len;
+    const char *key_buf;
+    struct cfl_list *head;
+    struct cfl_object *obj = NULL;
+    struct cfl_variant *val;
+    struct cfl_kvlist *kvlist;
+    struct cfl_kvpair *kvpair;
+    struct cfl_variant *var_attr;
+
+    if (context == CM_CONTEXT_OTEL_RESOURCE_ATTR) {
+        key_buf = "resource";
+        key_len = 8;
+    }
+    else if (context == CM_CONTEXT_OTEL_SCOPE_ATTR) {
+        key_buf = "scope";
+        key_len = 5;
+    }
+    else {
+        return NULL;
+    }
+
+    obj = record->cobj_record;
+    kvlist = obj->variant->data.as_kvlist;
+    cfl_list_foreach(head, &kvlist->list) {
+        kvpair = cfl_list_entry(head, struct cfl_kvpair, _head);
+
+        if (cfl_sds_len(kvpair->key) != key_len) {
+            continue;
+        }
+
+        if (strncmp(kvpair->key, key_buf, key_len) == 0) {
+            val = kvpair->val;
+            if (val->type != CFL_VARIANT_KVLIST) {
+                return NULL;
+            }
+
+            var_attr = otel_get_or_create_attributes(val->data.as_kvlist);
+            if (!var_attr) {
+                return NULL;
+            }
+
+            return var_attr;
+        }
+    }
+
+    return NULL;
+}
+
+static struct cfl_variant *otel_get_scope(struct flb_mp_chunk_record *record)
+{
+    struct cfl_list *head;
+    struct cfl_object *obj;
+    struct cfl_variant *val;
+    struct cfl_kvlist *kvlist;
+    struct cfl_kvpair *kvpair;
+
+    obj = record->cobj_record;
+    kvlist = obj->variant->data.as_kvlist;
+    cfl_list_foreach(head, &kvlist->list) {
+        kvpair = cfl_list_entry(head, struct cfl_kvpair, _head);
+
+        if (cfl_sds_len(kvpair->key) != 5) {
+            continue;
+        }
+
+        if (strncmp(kvpair->key, "scope", 5) == 0) {
+            val = kvpair->val;
+            if (val->type != CFL_VARIANT_KVLIST) {
+                return NULL;
+            }
+
+            return val;
+        }
+    }
+
+    return NULL;
+}
 int cm_logs_process(struct flb_processor_instance *ins,
                     struct content_modifier_ctx *ctx,
                     struct flb_mp_chunk_cobj *chunk_cobj,
@@ -602,11 +734,23 @@ int cm_logs_process(struct flb_processor_instance *ins,
                     int tag_len)
 {
     int ret = -1;
+    int record_type;
     struct flb_mp_chunk_record *record;
     struct cfl_object *obj = NULL;
+    struct cfl_object obj_static;
+    struct cfl_variant *var;
 
     /* Iterate records */
     while ((ret = flb_mp_chunk_cobj_record_next(chunk_cobj, &record)) == FLB_MP_CHUNK_RECORD_OK) {
+        obj = NULL;
+
+        /* Retrieve information about the record type */
+        ret = flb_log_event_decoder_get_record_type(&record->event, &record_type);
+        if (ret != 0) {
+            flb_plg_error(ctx->ins, "record has invalid event type");
+            continue;
+        }
+
         /* retrieve the target cfl object */
         if (ctx->context_type == CM_CONTEXT_LOG_METADATA) {
             obj = record->cobj_metadata;
@@ -614,6 +758,41 @@ int cm_logs_process(struct flb_processor_instance *ins,
         else if (ctx->context_type == CM_CONTEXT_LOG_BODY) {
             obj = record->cobj_record;
         }
+        else if (ctx->context_type == CM_CONTEXT_OTEL_RESOURCE_ATTR &&
+                 record_type == FLB_LOG_EVENT_GROUP_START) {
+            var = otel_get_attributes(CM_CONTEXT_OTEL_RESOURCE_ATTR, record);
+            if (!var) {
+                continue;
+            }
+
+            obj_static.type = CFL_VARIANT_KVLIST;
+            obj_static.variant = var;
+            obj = &obj_static;
+        }
+        else if (ctx->context_type == CM_CONTEXT_OTEL_SCOPE_ATTR &&
+                 record_type == FLB_LOG_EVENT_GROUP_START) {
+
+            var = otel_get_attributes(CM_CONTEXT_OTEL_SCOPE_ATTR, record);
+            if (!var) {
+                continue;
+            }
+
+            obj_static.type = CFL_VARIANT_KVLIST;
+            obj_static.variant = var;
+            obj = &obj_static;
+        }
+        else if ((ctx->context_type == CM_CONTEXT_OTEL_SCOPE_NAME || ctx->context_type == CM_CONTEXT_OTEL_SCOPE_VERSION) &&
+                 record_type == FLB_LOG_EVENT_GROUP_START) {
+
+            var = otel_get_scope(record);
+            obj_static.type = CFL_VARIANT_KVLIST;
+            obj_static.variant = var;
+            obj = &obj_static;
+        }
+
+        if (!obj) {
+            continue;
+        }
 
         /* the operation on top of the data type is unsupported */
         if (obj->variant->type != CFL_VARIANT_KVLIST) {
