@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -98,7 +98,7 @@ struct flb_aws_credentials *get_credentials_fn_http(struct flb_aws_provider
         return NULL;
     }
 
-    creds = flb_malloc(sizeof(struct flb_aws_credentials));
+    creds = flb_calloc(1, sizeof(struct flb_aws_credentials));
     if (!creds) {
         flb_errno();
         goto error;
@@ -170,7 +170,7 @@ void sync_fn_http(struct flb_aws_provider *provider) {
 
     flb_debug("[aws_credentials] Sync called on the http provider");
     /* remove async flag */
-    implementation->client->upstream->flags &= ~(FLB_IO_ASYNC);
+    flb_stream_disable_async_mode(&implementation->client->upstream->base);
 }
 
 void async_fn_http(struct flb_aws_provider *provider) {
@@ -178,7 +178,7 @@ void async_fn_http(struct flb_aws_provider *provider) {
 
     flb_debug("[aws_credentials] Async called on the http provider");
     /* add async flag */
-    implementation->client->upstream->flags |= FLB_IO_ASYNC;
+    flb_stream_enable_async_mode(&implementation->client->upstream->base);
 }
 
 void upstream_set_fn_http(struct flb_aws_provider *provider,
@@ -250,6 +250,8 @@ struct flb_aws_provider *flb_http_provider_create(struct flb_config *config,
         return NULL;
     }
 
+    pthread_mutex_init(&provider->lock, NULL);
+
     implementation = flb_calloc(1, sizeof(struct flb_aws_provider_http));
 
     if (!implementation) {
@@ -273,7 +275,7 @@ struct flb_aws_provider *flb_http_provider_create(struct flb_config *config,
         return NULL;
     }
 
-    upstream->net.connect_timeout = FLB_AWS_CREDENTIAL_NET_TIMEOUT;
+    upstream->base.net.connect_timeout = FLB_AWS_CREDENTIAL_NET_TIMEOUT;
 
     implementation->client = generator->create();
     if (!implementation->client) {
@@ -473,6 +475,10 @@ struct flb_aws_credentials *flb_parse_json_credentials(char *response,
                 t = &tokens[i];
                 current_token = &response[t->start];
                 len = t->end - t->start;
+                if (creds->access_key_id != NULL) {
+                    flb_error("Trying to double allocate access_key_id");
+                    goto error;
+                }
                 creds->access_key_id = flb_sds_create_len(current_token, len);
                 if (!creds->access_key_id) {
                     flb_errno();
@@ -486,6 +492,10 @@ struct flb_aws_credentials *flb_parse_json_credentials(char *response,
                 t = &tokens[i];
                 current_token = &response[t->start];
                 len = t->end - t->start;
+                if (creds->secret_access_key != NULL) {
+                    flb_error("Trying to double allocate secret_access_key");
+                    goto error;
+                }
                 creds->secret_access_key = flb_sds_create_len(current_token,
                                                               len);
                 if (!creds->secret_access_key) {
@@ -499,6 +509,10 @@ struct flb_aws_credentials *flb_parse_json_credentials(char *response,
                 t = &tokens[i];
                 current_token = &response[t->start];
                 len = t->end - t->start;
+                if (creds->session_token != NULL) {
+                    flb_error("Trying to double allocate session_token");
+                    goto error;
+                }
                 creds->session_token = flb_sds_create_len(current_token, len);
                 if (!creds->session_token) {
                     flb_errno();
