@@ -122,8 +122,8 @@ struct flb_kernel *flb_kernel_info()
         flb_errno();
         return NULL;
     }
-    kernel->minor = a;
-    kernel->major = b;
+    kernel->major = a;
+    kernel->minor = b;
     kernel->patch = c;
     kernel->s_version.data = flb_malloc(16);
 
