@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2015-2022 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -53,6 +53,12 @@ struct flb_config_map upstream_net[] = {
      "Prioritize IPv4 DNS results when trying to establish a connection"
     },
 
+    {
+     FLB_CONFIG_MAP_BOOL, "net.dns.prefer_ipv6", "false",
+     0, FLB_TRUE, offsetof(struct flb_net_setup, dns_prefer_ipv6),
+     "Prioritize IPv6 DNS results when trying to establish a connection"
+    },
+
     {
      FLB_CONFIG_MAP_BOOL, "net.keepalive", "true",
      0, FLB_TRUE, offsetof(struct flb_net_setup, keepalive),
@@ -165,6 +171,12 @@ struct mk_list *flb_upstream_get_config_map(struct flb_config *config)
                 upstream_net[config_index].def_value = "true";
             }
         }
+        if (config->dns_prefer_ipv6) {
+            if (strcmp(upstream_net[config_index].name,
+                       "net.dns.prefer_ipv6") == 0) {
+                upstream_net[config_index].def_value = "true";
+            }
+        }
     }
 
     config_map = flb_config_map_create(config, upstream_net);
@@ -442,10 +454,6 @@ struct flb_upstream *flb_upstream_create_url(struct flb_config *config,
  */
 static void shutdown_connection(struct flb_connection *u_conn)
 {
-    struct flb_upstream *u;
-
-    u = u_conn->upstream;
-
     if (u_conn->fd > 0 &&
         !u_conn->shutdown_flag) {
         shutdown(u_conn->fd, SHUT_RDWR);
