@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -70,6 +69,13 @@ int flb_ml_parser_builtin_create(struct flb_config *config)
         return -1;
     }
 
+    /* Ruby */
+    mlp = flb_ml_parser_ruby(config, NULL);
+    if (!mlp) {
+        flb_error("[multiline] could not init 'ruby' built-in parser");
+        return -1;
+    }
+
     /* Python */
     mlp = flb_ml_parser_python(config, NULL);
     if (!mlp) {
@@ -103,6 +109,9 @@ struct flb_ml_parser *flb_ml_parser_create(struct flb_config *ctx,
     if (match_str) {
         ml_parser->match_str = flb_sds_create(match_str);
         if (!ml_parser->match_str) {
+            if (ml_parser->name) {
+                flb_sds_destroy(ml_parser->name);
+            }
             flb_free(ml_parser);
             return NULL;
         }
@@ -217,7 +226,7 @@ struct flb_ml_parser_ins *flb_ml_parser_instance_create(struct flb_ml *ml,
     ret = flb_ml_group_add_parser(ml, ins);
     if (ret != 0) {
         flb_error("[multiline] could not register parser '%s' on "
-                  "multiline '%s 'group", ml->name);
+                  "multiline '%s 'group", name, ml->name);
         flb_free(ins);
         return NULL;
     }
